"""
    :codeauthor: asomers <asomers@gmail.com>
"""


import salt.modules.freebsd_sysctl as freebsd_sysctl
import salt.modules.systemd_service as systemd
from salt.exceptions import CommandExecutionError
from tests.support.helpers import dedent
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class FreeBSDSysctlTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.freebsd_sysctl module
    """

    def setup_loader_modules(self):
        return {freebsd_sysctl: {}, systemd: {}}

    def test_get(self):
        """
        Tests the return of get function
        """
        mock_cmd = MagicMock(return_value="1")
        with patch.dict(freebsd_sysctl.__salt__, {"cmd.run": mock_cmd}):
            self.assertEqual(freebsd_sysctl.get("vfs.usermount"), "1")

    def test_assign_failed(self):
        """
        Tests if the assignment was successful or not
        """
        cmd = {
            "pid": 1337,
            "retcode": 1,
            "stderr": "sysctl: unknown oid 'asef.esrhaseras.easr'",
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(freebsd_sysctl.__salt__, {"cmd.run_all": mock_cmd}):
            self.assertRaises(
                CommandExecutionError,
                freebsd_sysctl.assign,
                "asef.esrhaseras.easr",
                "backward",
            )

    def test_assign_success(self):
        """
        Tests the return of successful assign function
        """
        cmd = {
            "pid": 1337,
            "retcode": 0,
            "stderr": "",
            "stdout": "vfs.usermount: 0 -> 1",
        }
        ret = {"vfs.usermount": "1"}
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(freebsd_sysctl.__salt__, {"cmd.run_all": mock_cmd}):
            self.assertEqual(freebsd_sysctl.assign("vfs.usermount", 1), ret)

    def test_persist_no_conf_failure(self):
        """
        Tests adding of config file failure
        """
        asn_cmd = {
            "pid": 1337,
            "retcode": 1,
            "stderr": "sysctl: vfs.usermount=1: Operation not permitted",
            "stdout": "vfs.usermount: 1",
        }
        mock_asn_cmd = MagicMock(return_value=asn_cmd)
        cmd = "sysctl vfs.usermount=1"
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(
            freebsd_sysctl.__salt__,
            {"cmd.run_stdout": mock_cmd, "cmd.run_all": mock_asn_cmd},
        ):
            with patch("salt.utils.files.fopen", mock_open()) as m_open:
                self.assertRaises(
                    CommandExecutionError,
                    freebsd_sysctl.persist,
                    "net.ipv4.ip_forward",
                    1,
                    config=None,
                )

    def test_persist_updated(self):
        """
        Tests sysctl.conf success
        """
        cmd = {
            "pid": 1337,
            "retcode": 0,
            "stderr": "",
            "stdout": "vfs.usermount: 1 -> 1",
        }
        mock_cmd = MagicMock(return_value=cmd)

        with patch("salt.utils.files.fopen", mock_open()):
            with patch.dict(
                freebsd_sysctl.__salt__,
                {"cmd.run_all": mock_cmd},
            ):
                self.assertEqual(
                    freebsd_sysctl.persist("vfs.usermount", 1),
                    "Updated",
                )

    def test_persist_updated_tunable(self):
        """
        Tests loader.conf success
        """

        with patch("salt.utils.files.fopen", mock_open()):
            self.assertEqual(
                freebsd_sysctl.persist("vfs.usermount", 1, "/boot/loader.conf"),
                "Updated",
            )

    def test_show(self):
        """
        Tests the show function
        """
        # Mock just a small portion of the full "sysctl -ae" output, but be
        # sure to include a multi-line value.
        mock_cmd = MagicMock(
            return_value=dedent(
                """\
            kern.ostype=FreeBSD
            kern.osrelease=13.0-CURRENT
            kern.osrevision=199506
            kern.version=FreeBSD 13.0-CURRENT #246 r365916M: Thu Sep 24 09:17:12 MDT 2020
                user@host.domain:/usr/obj/usr/src/head
            /amd64.amd64/sys/GENERIC

            kern.maxvnodes=213989
            """,
                "\n",
            )
        )
        with patch.dict(freebsd_sysctl.__salt__, {"cmd.run": mock_cmd}):
            ret = freebsd_sysctl.show()
            self.assertEqual("FreeBSD", ret["kern.ostype"])
            self.assertEqual(
                dedent(
                    """\
                FreeBSD 13.0-CURRENT #246 r365916M: Thu Sep 24 09:17:12 MDT 2020
                    user@host.domain:/usr/obj/usr/src/head
                /amd64.amd64/sys/GENERIC
                """,
                    "\n",
                ),
                ret["kern.version"],
            )



"""
    salt.cli.spm
    ~~~~~~~~~~~~~

    Salt's spm cli parser.

.. versionadded:: 2015.8.0
"""


import salt.spm
import salt.utils.parsers as parsers
from salt.utils.verify import verify_env, verify_log


class SPM(parsers.SPMParser):
    """
    The cli parser object used to fire up the salt spm system.
    """

    def run(self):
        """
        Run the api
        """
        ui = salt.spm.SPMCmdlineInterface()
        self.parse_args()
        self.setup_logfile_logger()
        v_dirs = [
            self.config["spm_cache_dir"],
        ]
        verify_env(
            v_dirs,
            self.config["user"],
            root_dir=self.config["root_dir"],
        )
        verify_log(self.config)
        client = salt.spm.SPMClient(ui, self.config)
        client.run(self.args)



import pytest
import salt.states.win_dism as dism
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {dism: {}}


def test_capability_installed():
    """
    Test capability installed state
    """
    expected = {
        "comment": "Installed Capa2",
        "changes": {"capability": {"new": ["Capa2"]}, "retcode": 0},
        "name": "Capa2",
        "result": True,
    }

    mock_installed = MagicMock(side_effect=[["Capa1"], ["Capa1", "Capa2"]])
    mock_add = MagicMock(return_value={"retcode": 0})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_capabilities": mock_installed,
            "dism.add_capability": mock_add,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.capability_installed("Capa2", "somewhere", True)

            mock_installed.assert_called_with()
            mock_add.assert_called_once_with("Capa2", "somewhere", True, None, False)
            assert out == expected


def test_capability_installed_failure():
    """
    Test installing a capability which fails with DISM
    """
    expected = {
        "comment": "Failed to install Capa2: Failed",
        "changes": {},
        "name": "Capa2",
        "result": False,
    }

    mock_installed = MagicMock(side_effect=[["Capa1"], ["Capa1"]])
    mock_add = MagicMock(return_value={"retcode": 67, "stdout": "Failed"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_capabilities": mock_installed,
            "dism.add_capability": mock_add,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.capability_installed("Capa2", "somewhere", True)

            mock_installed.assert_called_with()
            mock_add.assert_called_once_with("Capa2", "somewhere", True, None, False)
            assert out == expected


def test_capability_installed_installed():
    """
    Test installing a capability already installed
    """
    expected = {
        "comment": "The capability Capa2 is already installed",
        "changes": {},
        "name": "Capa2",
        "result": True,
    }

    mock_installed = MagicMock(return_value=["Capa1", "Capa2"])
    mock_add = MagicMock()

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_capabilities": mock_installed,
            "dism.add_capability": mock_add,
        },
    ):

        with patch.dict(dism.__opts__, {"test": False}):
            out = dism.capability_installed("Capa2", "somewhere", True)

            mock_installed.assert_called_once_with()
            assert not mock_add.called
            assert out == expected


def test_capability_removed():
    """
    Test capability removed state
    """
    expected = {
        "comment": "Removed Capa2",
        "changes": {"capability": {"old": ["Capa2"]}, "retcode": 0},
        "name": "Capa2",
        "result": True,
    }

    mock_removed = MagicMock(side_effect=[["Capa1", "Capa2"], ["Capa1"]])
    mock_remove = MagicMock(return_value={"retcode": 0})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_capabilities": mock_removed,
            "dism.remove_capability": mock_remove,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.capability_removed("Capa2")

            mock_removed.assert_called_with()
            mock_remove.assert_called_once_with("Capa2", None, False)
            assert out == expected


def test_capability_removed_failure():
    """
    Test removing a capability which fails with DISM
    """
    expected = {
        "comment": "Failed to remove Capa2: Failed",
        "changes": {},
        "name": "Capa2",
        "result": False,
    }

    mock_removed = MagicMock(side_effect=[["Capa1", "Capa2"], ["Capa1", "Capa2"]])
    mock_remove = MagicMock(return_value={"retcode": 67, "stdout": "Failed"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_capabilities": mock_removed,
            "dism.remove_capability": mock_remove,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.capability_removed("Capa2")

            mock_removed.assert_called_with()
            mock_remove.assert_called_once_with("Capa2", None, False)
            assert out == expected


def test_capability_removed_removed():
    """
    Test removing a capability already removed
    """
    expected = {
        "comment": "The capability Capa2 is already removed",
        "changes": {},
        "name": "Capa2",
        "result": True,
    }

    mock_removed = MagicMock(return_value=["Capa1"])
    mock_remove = MagicMock()

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_capabilities": mock_removed,
            "dism.add_capability": mock_remove,
        },
    ):

        out = dism.capability_removed("Capa2", "somewhere", True)

        mock_removed.assert_called_once_with()
        assert not mock_remove.called
        assert out == expected


def test_feature_installed():
    """
    Test installing a feature with DISM
    """
    expected = {
        "comment": "Installed Feat2",
        "changes": {"feature": {"new": ["Feat2"]}, "retcode": 0},
        "name": "Feat2",
        "result": True,
    }

    mock_installed = MagicMock(side_effect=[["Feat1"], ["Feat1", "Feat2"]])
    mock_add = MagicMock(return_value={"retcode": 0})

    with patch.dict(
        dism.__salt__,
        {"dism.installed_features": mock_installed, "dism.add_feature": mock_add},
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.feature_installed("Feat2")

            mock_installed.assert_called_with()
            mock_add.assert_called_once_with(
                "Feat2", None, None, False, False, None, False
            )
            assert out == expected


def test_feature_installed_failure():
    """
    Test installing a feature which fails with DISM
    """
    expected = {
        "comment": "Failed to install Feat2: Failed",
        "changes": {},
        "name": "Feat2",
        "result": False,
    }

    mock_installed = MagicMock(side_effect=[["Feat1"], ["Feat1"]])
    mock_add = MagicMock(return_value={"retcode": 67, "stdout": "Failed"})

    with patch.dict(
        dism.__salt__,
        {"dism.installed_features": mock_installed, "dism.add_feature": mock_add},
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.feature_installed("Feat2")

            mock_installed.assert_called_with()
            mock_add.assert_called_once_with(
                "Feat2", None, None, False, False, None, False
            )
            assert out == expected


def test_feature_installed_installed():
    """
    Test installing a feature already installed
    """
    expected = {
        "comment": "The feature Feat1 is already installed",
        "changes": {},
        "name": "Feat1",
        "result": True,
    }

    mock_installed = MagicMock(side_effect=[["Feat1", "Feat2"], ["Feat1", "Feat2"]])
    mock_add = MagicMock()

    with patch.dict(
        dism.__salt__,
        {"dism.installed_features": mock_installed, "dism.add_feature": mock_add},
    ):

        out = dism.feature_installed("Feat1")

        mock_installed.assert_called_once_with()
        assert not mock_add.called
        assert out == expected


def test_feature_removed():
    """
    Test removing a feature with DISM
    """
    expected = {
        "comment": "Removed Feat2",
        "changes": {"feature": {"old": ["Feat2"]}, "retcode": 0},
        "name": "Feat2",
        "result": True,
    }

    mock_removed = MagicMock(side_effect=[["Feat1", "Feat2"], ["Feat1"]])
    mock_remove = MagicMock(return_value={"retcode": 0})

    with patch.dict(
        dism.__salt__,
        {"dism.installed_features": mock_removed, "dism.remove_feature": mock_remove},
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.feature_removed("Feat2")

            mock_removed.assert_called_with()
            mock_remove.assert_called_once_with("Feat2", False, None, False)
            assert out == expected


def test_feature_removed_failure():
    """
    Test removing a feature which fails with DISM
    """
    expected = {
        "comment": "Failed to remove Feat2: Failed",
        "changes": {},
        "name": "Feat2",
        "result": False,
    }

    mock_removed = MagicMock(side_effect=[["Feat1", "Feat2"], ["Feat1", "Feat2"]])
    mock_remove = MagicMock(return_value={"retcode": 67, "stdout": "Failed"})

    with patch.dict(
        dism.__salt__,
        {"dism.installed_features": mock_removed, "dism.remove_feature": mock_remove},
    ):
        with patch.dict(dism.__opts__, {"test": False}):

            out = dism.feature_removed("Feat2")

            mock_removed.assert_called_with()
            mock_remove.assert_called_once_with("Feat2", False, None, False)
            assert out == expected


def test_feature_removed_removed():
    """
    Test removing a feature already removed
    """
    expected = {
        "comment": "The feature Feat2 is already removed",
        "changes": {},
        "name": "Feat2",
        "result": True,
    }

    mock_removed = MagicMock(side_effect=[["Feat1"], ["Feat1"]])
    mock_remove = MagicMock()

    with patch.dict(
        dism.__salt__,
        {"dism.installed_features": mock_removed, "dism.remove_feature": mock_remove},
    ):

        out = dism.feature_removed("Feat2")

        mock_removed.assert_called_once_with()
        assert not mock_remove.called
        assert out == expected


def test_package_installed():
    """
    Test installing a package with DISM
    """
    expected = {
        "comment": "Installed Pack2",
        "changes": {"package": {"new": ["Pack2"]}, "retcode": 0},
        "name": "Pack2",
        "result": True,
    }

    mock_installed = MagicMock(side_effect=[["Pack1"], ["Pack1", "Pack2"]])
    mock_add = MagicMock(return_value={"retcode": 0})
    mock_info = MagicMock(return_value={"Package Identity": "Pack2"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_packages": mock_installed,
            "dism.add_package": mock_add,
            "dism.package_info": mock_info,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):
            with patch("os.path.exists"):

                out = dism.package_installed("Pack2")

                mock_installed.assert_called_with()
                mock_add.assert_called_once_with("Pack2", False, False, None, False)
                assert out == expected


def test_package_installed_failure():
    """
    Test installing a package which fails with DISM
    """
    expected = {
        "comment": "Failed to install Pack2: Failed",
        "changes": {},
        "name": "Pack2",
        "result": False,
    }

    mock_installed = MagicMock(side_effect=[["Pack1"], ["Pack1"]])
    mock_add = MagicMock(return_value={"retcode": 67, "stdout": "Failed"})
    mock_info = MagicMock(return_value={"Package Identity": "Pack2"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_packages": mock_installed,
            "dism.add_package": mock_add,
            "dism.package_info": mock_info,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):
            with patch("os.path.exists"):

                out = dism.package_installed("Pack2")

                mock_installed.assert_called_with()
                mock_add.assert_called_once_with("Pack2", False, False, None, False)
                assert out == expected


def test_package_installed_installed():
    """
    Test installing a package already installed
    """
    expected = {
        "comment": "The package Pack2 is already installed: Pack2",
        "changes": {},
        "name": "Pack2",
        "result": True,
    }

    mock_installed = MagicMock(side_effect=[["Pack1", "Pack2"], ["Pack1", "Pack2"]])
    mock_add = MagicMock()
    mock_info = MagicMock(return_value={"Package Identity": "Pack2"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_packages": mock_installed,
            "dism.add_package": mock_add,
            "dism.package_info": mock_info,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):
            with patch("os.path.exists"):

                out = dism.package_installed("Pack2")

                mock_installed.assert_called_once_with()
                assert not mock_add.called
                assert out == expected


def test_package_removed():
    """
    Test removing a package with DISM
    """
    expected = {
        "comment": "Removed Pack2",
        "changes": {"package": {"old": ["Pack2"]}, "retcode": 0},
        "name": "Pack2",
        "result": True,
    }

    mock_removed = MagicMock(side_effect=[["Pack1", "Pack2"], ["Pack1"]])
    mock_remove = MagicMock(return_value={"retcode": 0})
    mock_info = MagicMock(return_value={"Package Identity": "Pack2"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_packages": mock_removed,
            "dism.remove_package": mock_remove,
            "dism.package_info": mock_info,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):
            with patch("os.path.exists"):

                out = dism.package_removed("Pack2")

                mock_removed.assert_called_with()
                mock_remove.assert_called_once_with("Pack2", None, False)
                assert out == expected


def test_package_removed_failure():
    """
    Test removing a package which fails with DISM
    """
    expected = {
        "comment": "Failed to remove Pack2: Failed",
        "changes": {},
        "name": "Pack2",
        "result": False,
    }

    mock_removed = MagicMock(side_effect=[["Pack1", "Pack2"], ["Pack1", "Pack2"]])
    mock_remove = MagicMock(return_value={"retcode": 67, "stdout": "Failed"})
    mock_info = MagicMock(return_value={"Package Identity": "Pack2"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_packages": mock_removed,
            "dism.remove_package": mock_remove,
            "dism.package_info": mock_info,
        },
    ):
        with patch.dict(dism.__opts__, {"test": False}):
            with patch("os.path.exists"):

                out = dism.package_removed("Pack2")

                mock_removed.assert_called_with()
                mock_remove.assert_called_once_with("Pack2", None, False)
                assert out == expected


def test_package_removed_removed():
    """
    Test removing a package already removed
    """
    expected = {
        "comment": "The package Pack2 is already removed",
        "changes": {},
        "name": "Pack2",
        "result": True,
    }

    mock_removed = MagicMock(side_effect=[["Pack1"], ["Pack1"]])
    mock_remove = MagicMock()
    mock_info = MagicMock(return_value={"Package Identity": "Pack2"})

    with patch.dict(
        dism.__salt__,
        {
            "dism.installed_packages": mock_removed,
            "dism.remove_package": mock_remove,
            "dism.package_info": mock_info,
        },
    ):

        with patch.dict(dism.__opts__, {"test": False}):
            with patch("os.path.exists"):
                out = dism.package_removed("Pack2")

                mock_removed.assert_called_once_with()
                assert not mock_remove.called
                assert out == expected



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""


import salt.modules.deb_apache as deb_apache
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DebApacheTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.deb_apache
    """

    def setup_loader_modules(self):
        return {deb_apache: {}}

    # 'check_site_enabled' function tests: 3

    def test_check_site_enabled(self):
        """
        Test if the specific Site symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(return_value=True)):
            self.assertTrue(deb_apache.check_site_enabled("saltstack.com"))

    def test_check_site_enabled_default(self):
        """
        Test if the specific Site symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(side_effect=[False, True])):
            self.assertTrue(deb_apache.check_site_enabled("default"))

    def test_check_site_enabled_false(self):
        """
        Test if the specific Site symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(return_value=False)):
            self.assertFalse(deb_apache.check_site_enabled("saltstack.com"))

    # 'a2ensite' function tests: 4

    def test_a2ensite_notfound(self):
        """
        Test if it runs a2ensite for the given site.
        """
        mock = MagicMock(return_value=1)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2ensite("saltstack.com"),
                {
                    "Name": "Apache2 Enable Site",
                    "Site": "saltstack.com",
                    "Status": "Site saltstack.com Not found",
                },
            )

    def test_a2ensite_enabled(self):
        """
        Test if it runs a2ensite for the given site.
        """
        mock = MagicMock(return_value=0)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2ensite("saltstack.com"),
                {
                    "Name": "Apache2 Enable Site",
                    "Site": "saltstack.com",
                    "Status": "Site saltstack.com enabled",
                },
            )

    def test_a2ensite(self):
        """
        Test if it runs a2ensite for the given site.
        """
        mock = MagicMock(return_value=2)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2ensite("saltstack.com"),
                {"Name": "Apache2 Enable Site", "Site": "saltstack.com", "Status": 2},
            )

    def test_a2ensite_exception(self):
        """
        Test if it runs a2ensite for the given site.
        """
        mock = MagicMock(side_effect=Exception("error"))
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(str(deb_apache.a2ensite("saltstack.com")), "error")

    # 'a2dissite' function tests: 4

    def test_a2dissite_notfound(self):
        """
        Test if it runs a2dissite for the given site.
        """
        mock = MagicMock(return_value=256)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2dissite("saltstack.com"),
                {
                    "Name": "Apache2 Disable Site",
                    "Site": "saltstack.com",
                    "Status": "Site saltstack.com Not found",
                },
            )

    def test_a2dissite_disabled(self):
        """
        Test if it runs a2dissite for the given site.
        """
        mock = MagicMock(return_value=0)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2dissite("saltstack.com"),
                {
                    "Name": "Apache2 Disable Site",
                    "Site": "saltstack.com",
                    "Status": "Site saltstack.com disabled",
                },
            )

    def test_a2dissite(self):
        """
        Test if it runs a2dissite for the given site.
        """
        mock = MagicMock(return_value=2)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2dissite("saltstack.com"),
                {"Name": "Apache2 Disable Site", "Site": "saltstack.com", "Status": 2},
            )

    def test_a2dissite_exception(self):
        """
        Test if it runs a2dissite for the given site.
        """
        mock = MagicMock(side_effect=Exception("error"))
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(str(deb_apache.a2dissite("saltstack.com")), "error")

    # 'check_mod_enabled' function tests: 2

    def test_check_mod_enabled(self):
        """
        Test if the specific mod symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(return_value=True)):
            self.assertTrue(deb_apache.check_mod_enabled("status.conf"))

    def test_check_mod_enabled_false(self):
        """
        Test if the specific mod symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(return_value=False)):
            self.assertFalse(deb_apache.check_mod_enabled("status.conf"))

    # 'a2enmod' function tests: 4

    def test_a2enmod_notfound(self):
        """
        Test if it runs a2enmod for the given module.
        """
        mock = MagicMock(return_value=1)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2enmod("vhost_alias"),
                {
                    "Name": "Apache2 Enable Mod",
                    "Mod": "vhost_alias",
                    "Status": "Mod vhost_alias Not found",
                },
            )

    def test_a2enmod_enabled(self):
        """
        Test if it runs a2enmod for the given module.
        """
        mock = MagicMock(return_value=0)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2enmod("vhost_alias"),
                {
                    "Name": "Apache2 Enable Mod",
                    "Mod": "vhost_alias",
                    "Status": "Mod vhost_alias enabled",
                },
            )

    def test_a2enmod(self):
        """
        Test if it runs a2enmod for the given module.
        """
        mock = MagicMock(return_value=2)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2enmod("vhost_alias"),
                {"Name": "Apache2 Enable Mod", "Mod": "vhost_alias", "Status": 2},
            )

    def test_a2enmod_exception(self):
        """
        Test if it runs a2enmod for the given module.
        """
        mock = MagicMock(side_effect=Exception("error"))
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(str(deb_apache.a2enmod("vhost_alias")), "error")

    # 'a2dismod' function tests: 4

    def test_a2dismod_notfound(self):
        """
        Test if it runs a2dismod for the given module.
        """
        mock = MagicMock(return_value=256)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2dismod("vhost_alias"),
                {
                    "Name": "Apache2 Disable Mod",
                    "Mod": "vhost_alias",
                    "Status": "Mod vhost_alias Not found",
                },
            )

    def test_a2dismod_disabled(self):
        """
        Test if it runs a2dismod for the given module.
        """
        mock = MagicMock(return_value=0)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2dismod("vhost_alias"),
                {
                    "Name": "Apache2 Disable Mod",
                    "Mod": "vhost_alias",
                    "Status": "Mod vhost_alias disabled",
                },
            )

    def test_a2dismod(self):
        """
        Test if it runs a2dismod for the given module.
        """
        mock = MagicMock(return_value=2)
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(
                deb_apache.a2dismod("vhost_alias"),
                {"Name": "Apache2 Disable Mod", "Mod": "vhost_alias", "Status": 2},
            )

    def test_a2dismod_exception(self):
        """
        Test if it runs a2dismod for the given module.
        """
        mock = MagicMock(side_effect=Exception("error"))
        with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
            self.assertEqual(str(deb_apache.a2dismod("vhost_alias")), "error")

    # 'check_conf_enabled' function tests: 2

    def test_check_conf_enabled(self):
        """
        Test if the specific conf symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(return_value=True)):
            self.assertTrue(deb_apache.check_conf_enabled("security.conf"))

    def test_check_conf_enabled_false(self):
        """
        Test if the specific conf symlink is enabled.
        """
        with patch("os.path.islink", MagicMock(return_value=False)):
            self.assertFalse(deb_apache.check_conf_enabled("security.conf"))

    # 'a2enconf' function tests: 4

    def test_a2enconf_notfound(self):
        """
        Test if it runs a2enconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2enconf")):
            mock = MagicMock(return_value=1)
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(
                    deb_apache.a2enconf("security"),
                    {
                        "Name": "Apache2 Enable Conf",
                        "Conf": "security",
                        "Status": "Conf security Not found",
                    },
                )

    def test_a2enconf_enabled(self):
        """
        Test if it runs a2enconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2enconf")):
            mock = MagicMock(return_value=0)
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(
                    deb_apache.a2enconf("security"),
                    {
                        "Name": "Apache2 Enable Conf",
                        "Conf": "security",
                        "Status": "Conf security enabled",
                    },
                )

    def test_a2enconf(self):
        """
        Test if it runs a2enconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2enconf")):
            mock = MagicMock(return_value=2)
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(
                    deb_apache.a2enconf("security"),
                    {"Name": "Apache2 Enable Conf", "Conf": "security", "Status": 2},
                )

    def test_a2enconf_exception(self):
        """
        Test if it runs a2enconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2enconf")):
            mock = MagicMock(side_effect=Exception("error"))
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(str(deb_apache.a2enconf("security")), "error")

    # 'a2disconf' function tests: 4

    def test_a2disconf_notfound(self):
        """
        Test if it runs a2disconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2disconf")):
            mock = MagicMock(return_value=256)
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(
                    deb_apache.a2disconf("security"),
                    {
                        "Name": "Apache2 Disable Conf",
                        "Conf": "security",
                        "Status": "Conf security Not found",
                    },
                )

    def test_a2disconf_disabled(self):
        """
        Test if it runs a2disconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2disconf")):
            mock = MagicMock(return_value=0)
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(
                    deb_apache.a2disconf("security"),
                    {
                        "Name": "Apache2 Disable Conf",
                        "Conf": "security",
                        "Status": "Conf security disabled",
                    },
                )

    def test_a2disconf(self):
        """
        Test if it runs a2disconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2disconf")):
            mock = MagicMock(return_value=2)
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(
                    deb_apache.a2disconf("security"),
                    {"Name": "Apache2 Disable Conf", "Conf": "security", "Status": 2},
                )

    def test_a2disconf_exception(self):
        """
        Test if it runs a2disconf for the given conf.
        """
        with patch("salt.utils.path.which", MagicMock(return_value="a2disconf")):
            mock = MagicMock(side_effect=Exception("error"))
            with patch.dict(deb_apache.__salt__, {"cmd.retcode": mock}):
                self.assertEqual(str(deb_apache.a2disconf("security")), "error")



import textwrap

from salt.modules import devinfo
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase


class DevinfoTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {devinfo: {}}

    def test_udev(self):
        self.assertEqual(devinfo._udev({"A": {"B": 1}}, "a.b"), 1)
        self.assertEqual(devinfo._udev({"A": {"B": 1}}, "A.B"), 1)
        self.assertEqual(devinfo._udev({"A": {"B": 1}}, "a.c"), "n/a")
        self.assertEqual(devinfo._udev({"A": [1, 2]}, "a.b"), "n/a")
        self.assertEqual(devinfo._udev({"A": {"B": 1}}, ""), {"A": {"B": 1}})

    def test_match(self):
        self.assertTrue(devinfo._match({"A": {"B": 1}}, {"a.b": 1}))
        self.assertFalse(devinfo._match({"A": {"B": 1}}, {"a.b": 2}))
        self.assertTrue(devinfo._match({"A": {"B": 1}}, {"a.b": [1, 2]}))
        self.assertFalse(devinfo._match({"A": {"B": 1}}, {"a.b": [2, 3]}))
        self.assertTrue(devinfo._match({"A": {"B": [1, 2]}}, {"a.b": 1}))
        self.assertTrue(devinfo._match({"A": {"B": [1, 2]}}, {"a.b": [1, 3]}))
        self.assertFalse(devinfo._match({"A": {"B": [1, 2]}}, {"a.b": [3, 4]}))
        self.assertTrue(devinfo._match({"A": 1}, {}))

    def test_devices(self):
        cdrom = {
            "S": ["dvd", "cdrom"],
            "E": {"ID_BUS": "ata"},
        }
        usb = {
            "E": {"ID_BUS": "usb"},
        }
        hd = {
            "E": {"ID_BUS": "ata"},
        }

        with patch.dict(
            devinfo.__salt__,
            {"udev.info": lambda d: {"sda": hd, "sdb": usb, "sr0": cdrom}[d]},
        ), patch.dict(devinfo.__grains__, {"disks": ["sda", "sdb", "sr0"]}):
            self.assertEqual(devinfo.filter_({"e.id_bus": "ata"}, {}), ["sda", "sr0"])
            self.assertEqual(devinfo.filter_({"e.id_bus": "usb"}, {}), ["sdb"])
            self.assertEqual(
                devinfo.filter_({"e.id_bus": "ata"}, {"s": ["cdrom"]}), ["sda"]
            )

    def test__hwinfo_parse_short(self):
        hwinfo = textwrap.dedent(
            """
            cpu:
                                   QEMU Virtual CPU version 2.5+, 3591 MHz
            keyboard:
              /dev/input/event0    AT Translated Set 2 keyboard
            mouse:
              /dev/input/mice      VirtualPS/2 VMware VMMouse
              /dev/input/mice      VirtualPS/2 VMware VMMouse
            graphics card:
                                   VGA compatible controller
            storage:
                                   Floppy disk controller
                                   Red Hat Qemu virtual machine
            network:
              ens3                 Virtio Ethernet Card 0
            network interface:
              lo                   Loopback network interface
              ens3                 Ethernet network interface
            disk:
              /dev/fd0             Disk
              /dev/sda             QEMU HARDDISK
            cdrom:
              /dev/sr0             QEMU DVD-ROM
            floppy:
              /dev/fd0             Floppy Disk
            bios:
                                   BIOS
            bridge:
                                   Red Hat Qemu virtual machine
                                   Red Hat Qemu virtual machine
                                   Red Hat Qemu virtual machine
            memory:
                                   Main Memory
            unknown:
                                   FPU
                                   DMA controller
                                   PIC
                                   Keyboard controller
              /dev/lp0             Parallel controller
                                   PS/2 Controller
                                   Red Hat Virtio network device
              /dev/ttyS0           16550A
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_short(hwinfo),
            {
                "cpu": {0: "QEMU Virtual CPU version 2.5+, 3591 MHz"},
                "keyboard": {"/dev/input/event0": "AT Translated Set 2 keyboard"},
                "mouse": {"/dev/input/mice": "VirtualPS/2 VMware VMMouse"},
                "graphics card": {0: "VGA compatible controller"},
                "storage": {
                    0: "Floppy disk controller",
                    1: "Red Hat Qemu virtual machine",
                },
                "network": {"ens3": "Virtio Ethernet Card 0"},
                "network interface": {
                    "lo": "Loopback network interface",
                    "ens3": "Ethernet network interface",
                },
                "disk": {"/dev/fd0": "Disk", "/dev/sda": "QEMU HARDDISK"},
                "cdrom": {"/dev/sr0": "QEMU DVD-ROM"},
                "floppy": {"/dev/fd0": "Floppy Disk"},
                "bios": {0: "BIOS"},
                "bridge": {
                    0: "Red Hat Qemu virtual machine",
                    1: "Red Hat Qemu virtual machine",
                    2: "Red Hat Qemu virtual machine",
                },
                "memory": {0: "Main Memory"},
                "unknown": {
                    0: "FPU",
                    1: "DMA controller",
                    2: "PIC",
                    3: "Keyboard controller",
                    "/dev/lp0": "Parallel controller",
                    4: "PS/2 Controller",
                    5: "Red Hat Virtio network device",
                    "/dev/ttyS0": "16550A",
                },
            },
        )

    def test__hwinfo_parse_full_floppy(self):
        hwinfo = textwrap.dedent(
            """
            01: None 00.0: 0102 Floppy disk controller
              [Created at floppy.112]
              Unique ID: rdCR.3wRL2_g4d2B
              Hardware Class: storage
              Model: "Floppy disk controller"
              I/O Port: 0x3f2 (rw)
              I/O Ports: 0x3f4-0x3f5 (rw)
              I/O Port: 0x3f7 (rw)
              DMA: 2
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            02: Floppy 00.0: 10603 Floppy Disk
              [Created at floppy.127]
              Unique ID: sPPV.oZ89vuho4Y3
              Parent ID: rdCR.3wRL2_g4d2B
              Hardware Class: floppy
              Model: "Floppy Disk"
              Device File: /dev/fd0
              Size: 3.5 ''
              Config Status: cfg=new, avail=yes, need=no, active=unknown
              Size: 5760 sectors a 512 bytes
              Capacity: 0 GB (2949120 bytes)
              Drive status: no medium
              Config Status: cfg=new, avail=yes, need=no, active=unknown
              Attached to: #1 (Floppy disk controller)
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "01": {
                    "None 00.0": "0102 Floppy disk controller",
                    "Note": "Created at floppy.112",
                    "Unique ID": "rdCR.3wRL2_g4d2B",
                    "Hardware Class": "storage",
                    "Model": "Floppy disk controller",
                    "I/O Ports": ["0x3f2 (rw)", "0x3f4-0x3f5 (rw)", "0x3f7 (rw)"],
                    "DMA": "2",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "02": {
                    "Floppy 00.0": "10603 Floppy Disk",
                    "Note": "Created at floppy.127",
                    "Unique ID": "sPPV.oZ89vuho4Y3",
                    "Parent ID": "rdCR.3wRL2_g4d2B",
                    "Hardware Class": "floppy",
                    "Model": "Floppy Disk",
                    "Device File": "/dev/fd0",
                    "Size": ["3.5 ''", "5760 sectors a 512 bytes"],
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                    "Capacity": "0 GB (2949120 bytes)",
                    "Drive status": "no medium",
                    "Attached to": {"Handle": "#1 (Floppy disk controller)"},
                },
            },
        )

    def test__hwinfo_parse_full_bios(self):
        hwinfo = textwrap.dedent(
            """
            03: None 00.0: 10105 BIOS
              [Created at bios.186]
              Unique ID: rdCR.lZF+r4EgHp4
              Hardware Class: bios
              BIOS Keyboard LED Status:
                Scroll Lock: off
                Num Lock: off
                Caps Lock: off
              Serial Port 0: 0x3f8
              Parallel Port 0: 0x378
              Base Memory: 639 kB
              PnP BIOS: @@@0000
              MP spec rev 1.4 info:
                OEM id: "BOCHSCPU"
                Product id: "0.1"
                1 CPUs (0 disabled)
              BIOS32 Service Directory Entry: 0xfd2b0
              SMBIOS Version: 2.8
              BIOS Info: #0
                Vendor: "SeaBIOS"
                Version: "rel-1.12.1-0-ga5cab58e9a3f-prebuilt.qemu.org"
                Date: "04/01/2014"
                Start Address: 0xe8000
                ROM Size: 64 kB
                Features: 0x04000000000000000008
              System Info: #256
                Manufacturer: "QEMU"
                Product: "Standard PC (i440FX + PIIX, 1996)"
                Version: "pc-i440fx-4.0"
                UUID: undefined
                Wake-up: 0x06 (Power Switch)
              Chassis Info: #768
                Manufacturer: "QEMU"
                Version: "pc-i440fx-4.0"
                Type: 0x01 (Other)
                Bootup State: 0x03 (Safe)
                Power Supply State: 0x03 (Safe)
                Thermal State: 0x03 (Safe)
                Security Status: 0x02 (Unknown)
              Processor Info: #1024
                Socket: "CPU 0"
                Socket Type: 0x01 (Other)
                Socket Status: Populated
                Type: 0x03 (CPU)
                Family: 0x01 (Other)
                Manufacturer: "QEMU"
                Version: "pc-i440fx-4.0"
                Processor ID: 0x078bfbfd00000663
                Status: 0x01 (Enabled)
                Max. Speed: 2000 MHz
                Current Speed: 2000 MHz
              Physical Memory Array: #4096
                Use: 0x03 (System memory)
                Location: 0x01 (Other)
                Slots: 1
                Max. Size: 1 GB
                ECC: 0x06 (Multi-bit)
              Memory Device: #4352
                Location: "DIMM 0"
                Manufacturer: "QEMU"
                Memory Array: #4096
                Form Factor: 0x09 (DIMM)
                Type: 0x07 (RAM)
                Type Detail: 0x0002 (Other)
                Data Width: 0 bits
                 Size: 1 GB
              Memory Array Mapping: #4864
                Memory Array: #4096
                Partition Width: 1
                Start Address: 0x00000000
                End Address: 0x40000000
              Type 32 Record: #8192
                Data 00: 20 0b 00 20 00 00 00 00 00 00 00
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "03": {
                    "None 00.0": "10105 BIOS",
                    "Note": "Created at bios.186",
                    "Unique ID": "rdCR.lZF+r4EgHp4",
                    "Hardware Class": "bios",
                    "BIOS Keyboard LED Status": {
                        "Scroll Lock": "off",
                        "Num Lock": "off",
                        "Caps Lock": "off",
                    },
                    "Serial Port 0": "0x3f8",
                    "Parallel Port 0": "0x378",
                    "Base Memory": "639 kB",
                    "PnP BIOS": "@@@0000",
                    "MP spec rev 1.4 info": {
                        "OEM id": "BOCHSCPU",
                        "Product id": "0.1",
                        "Note": "1 CPUs (0 disabled)",
                    },
                    "BIOS32 Service Directory Entry": "0xfd2b0",
                    "SMBIOS Version": "2.8",
                    "BIOS Info": {
                        "Handle": "#0",
                        "Vendor": "SeaBIOS",
                        "Version": "rel-1.12.1-0-ga5cab58e9a3f-prebuilt.qemu.org",
                        "Date": "04/01/2014",
                        "Start Address": "0xe8000",
                        "ROM Size": "64 kB",
                        "Features": ["0x04000000000000000008"],
                    },
                    "System Info": {
                        "Handle": "#256",
                        "Manufacturer": "QEMU",
                        "Product": "Standard PC (i440FX + PIIX, 1996)",
                        "Version": "pc-i440fx-4.0",
                        "UUID": "undefined",
                        "Wake-up": "0x06 (Power Switch)",
                    },
                    "Chassis Info": {
                        "Handle": "#768",
                        "Manufacturer": "QEMU",
                        "Version": "pc-i440fx-4.0",
                        "Type": "0x01 (Other)",
                        "Bootup State": "0x03 (Safe)",
                        "Power Supply State": "0x03 (Safe)",
                        "Thermal State": "0x03 (Safe)",
                        "Security Status": "0x02 (Unknown)",
                    },
                    "Processor Info": {
                        "Handle": "#1024",
                        "Socket": "CPU 0",
                        "Socket Type": "0x01 (Other)",
                        "Socket Status": "Populated",
                        "Type": "0x03 (CPU)",
                        "Family": "0x01 (Other)",
                        "Manufacturer": "QEMU",
                        "Version": "pc-i440fx-4.0",
                        "Processor ID": "0x078bfbfd00000663",
                        "Status": "0x01 (Enabled)",
                        "Max. Speed": "2000 MHz",
                        "Current Speed": "2000 MHz",
                    },
                    "Physical Memory Array": {
                        "Handle": "#4096",
                        "Use": "0x03 (System memory)",
                        "Location": "0x01 (Other)",
                        "Slots": "1",
                        "Max. Size": "1 GB",
                        "ECC": "0x06 (Multi-bit)",
                    },
                    "Memory Device": {
                        "Handle": "#4352",
                        "Location": "DIMM 0",
                        "Manufacturer": "QEMU",
                        "Memory Array": {"Handle": "#4096"},
                        "Form Factor": "0x09 (DIMM)",
                        "Type": "0x07 (RAM)",
                        "Type Detail": "0x0002 (Other)",
                        "Data Width": "0 bits",
                        "Size": "1 GB",
                    },
                    "Memory Array Mapping": {
                        "Handle": "#4864",
                        "Memory Array": {"Handle": "#4096"},
                        "Partition Width": "1",
                        "Start Address": "0x00000000",
                        "End Address": "0x40000000",
                    },
                    "Type 32 Record": {
                        "Handle": "#8192",
                        "Data 00": "20 0b 00 20 00 00 00 00 00 00 00",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_system(self):
        hwinfo = textwrap.dedent(
            """
            04: None 00.0: 10107 System
              [Created at sys.64]
              Unique ID: rdCR.n_7QNeEnh23
              Hardware Class: system
              Model: "System"
              Formfactor: "desktop"
              Driver Info #0:
                Driver Status: thermal,fan are not active
                Driver Activation Cmd: "modprobe thermal; modprobe fan"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "04": {
                    "None 00.0": "10107 System",
                    "Note": "Created at sys.64",
                    "Unique ID": "rdCR.n_7QNeEnh23",
                    "Hardware Class": "system",
                    "Model": "System",
                    "Formfactor": "desktop",
                    "Driver Info #0": {
                        "Driver Status": "thermal,fan are not active",
                        "Driver Activation Cmd": "modprobe thermal; modprobe fan",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_unknown(self):
        hwinfo = textwrap.dedent(
            """
            05: None 00.0: 10104 FPU
              [Created at misc.191]
              Unique ID: rdCR.EMpH5pjcahD
              Hardware Class: unknown
              Model: "FPU"
              I/O Ports: 0xf0-0xff (rw)
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            06: None 00.0: 0801 DMA controller (8237)
              [Created at misc.205]
              Unique ID: rdCR.f5u1ucRm+H9
              Hardware Class: unknown
              Model: "DMA controller"
              I/O Ports: 0x00-0xcf7 (rw)
              I/O Ports: 0xc0-0xdf (rw)
              I/O Ports: 0x80-0x8f (rw)
              DMA: 4
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            07: None 00.0: 0800 PIC (8259)
              [Created at misc.218]
              Unique ID: rdCR.8uRK7LxiIA2
              Hardware Class: unknown
              Model: "PIC"
              I/O Ports: 0x20-0x21 (rw)
              I/O Ports: 0xa0-0xa1 (rw)
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            08: None 00.0: 0900 Keyboard controller
              [Created at misc.250]
              Unique ID: rdCR.9N+EecqykME
              Hardware Class: unknown
              Model: "Keyboard controller"
              I/O Port: 0x60 (rw)
              I/O Port: 0x64 (rw)
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            09: None 00.0: 0701 Parallel controller (SPP)
              [Created at misc.261]
              Unique ID: YMnp.ecK7NLYWZ5D
              Hardware Class: unknown
              Model: "Parallel controller"
              Device File: /dev/lp0
              I/O Ports: 0x378-0x37a (rw)
              I/O Ports: 0x37b-0x37f (rw)
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            10: None 00.0: 10400 PS/2 Controller
              [Created at misc.303]
              Unique ID: rdCR.DziBbWO85o5
              Hardware Class: unknown
              Model: "PS/2 Controller"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "05": {
                    "None 00.0": "10104 FPU",
                    "Note": "Created at misc.191",
                    "Unique ID": "rdCR.EMpH5pjcahD",
                    "Hardware Class": "unknown",
                    "Model": "FPU",
                    "I/O Ports": "0xf0-0xff (rw)",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "06": {
                    "None 00.0": "0801 DMA controller (8237)",
                    "Note": "Created at misc.205",
                    "Unique ID": "rdCR.f5u1ucRm+H9",
                    "Hardware Class": "unknown",
                    "Model": "DMA controller",
                    "I/O Ports": [
                        "0x00-0xcf7 (rw)",
                        "0xc0-0xdf (rw)",
                        "0x80-0x8f (rw)",
                    ],
                    "DMA": "4",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "07": {
                    "None 00.0": "0800 PIC (8259)",
                    "Note": "Created at misc.218",
                    "Unique ID": "rdCR.8uRK7LxiIA2",
                    "Hardware Class": "unknown",
                    "Model": "PIC",
                    "I/O Ports": ["0x20-0x21 (rw)", "0xa0-0xa1 (rw)"],
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "08": {
                    "None 00.0": "0900 Keyboard controller",
                    "Note": "Created at misc.250",
                    "Unique ID": "rdCR.9N+EecqykME",
                    "Hardware Class": "unknown",
                    "Model": "Keyboard controller",
                    "I/O Ports": ["0x60 (rw)", "0x64 (rw)"],
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "09": {
                    "None 00.0": "0701 Parallel controller (SPP)",
                    "Note": "Created at misc.261",
                    "Unique ID": "YMnp.ecK7NLYWZ5D",
                    "Hardware Class": "unknown",
                    "Model": "Parallel controller",
                    "Device File": "/dev/lp0",
                    "I/O Ports": ["0x378-0x37a (rw)", "0x37b-0x37f (rw)"],
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "10": {
                    "None 00.0": "10400 PS/2 Controller",
                    "Note": "Created at misc.303",
                    "Unique ID": "rdCR.DziBbWO85o5",
                    "Hardware Class": "unknown",
                    "Model": "PS/2 Controller",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_memory(self):
        hwinfo = textwrap.dedent(
            """
            12: None 00.0: 10102 Main Memory
              [Created at memory.74]
              Unique ID: rdCR.CxwsZFjVASF
              Hardware Class: memory
              Model: "Main Memory"
              Memory Range: 0x00000000-0x3cefffff (rw)
              Memory Size: 960 MB
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "12": {
                    "None 00.0": "10102 Main Memory",
                    "Note": "Created at memory.74",
                    "Unique ID": "rdCR.CxwsZFjVASF",
                    "Hardware Class": "memory",
                    "Model": "Main Memory",
                    "Memory Range": "0x00000000-0x3cefffff (rw)",
                    "Memory Size": "960 MB",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_bridge(self):
        hwinfo = textwrap.dedent(
            """
            13: PCI 01.0: 0601 ISA bridge
              [Created at pci.386]
              Unique ID: vSkL.ucdhKwLeeAA
              SysFS ID: /devices/pci0000:00/0000:00:01.0
              SysFS BusID: 0000:00:01.0
              Hardware Class: bridge
              Model: "Red Hat Qemu virtual machine"
              Vendor: pci 0x8086 "Intel Corporation"
              Device: pci 0x7000 "82371SB PIIX3 ISA [Natoma/Triton II]"
              SubVendor: pci 0x1af4 "Red Hat, Inc."
              SubDevice: pci 0x1100 "Qemu virtual machine"
              Module Alias: "pci:v00008086d00007000sv00001AF4sd00001100bc06sc01i00"
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            14: PCI 00.0: 0600 Host bridge
              [Created at pci.386]
              Unique ID: qLht.YeL3TKDjrxE
              SysFS ID: /devices/pci0000:00/0000:00:00.0
              SysFS BusID: 0000:00:00.0
              Hardware Class: bridge
              Model: "Red Hat Qemu virtual machine"
              Vendor: pci 0x8086 "Intel Corporation"
              Device: pci 0x1237 "440FX - 82441FX PMC [Natoma]"
              SubVendor: pci 0x1af4 "Red Hat, Inc."
              SubDevice: pci 0x1100 "Qemu virtual machine"
              Revision: 0x02
              Module Alias: "pci:v00008086d00001237sv00001AF4sd00001100bc06sc00i00"
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            15: PCI 01.3: 0680 Bridge
              [Created at pci.386]
              Unique ID: VRCs.M9Cc8lcQjE2
              SysFS ID: /devices/pci0000:00/0000:00:01.3
              SysFS BusID: 0000:00:01.3
              Hardware Class: bridge
              Model: "Red Hat Qemu virtual machine"
              Vendor: pci 0x8086 "Intel Corporation"
              Device: pci 0x7113 "82371AB/EB/MB PIIX4 ACPI"
              SubVendor: pci 0x1af4 "Red Hat, Inc."
              SubDevice: pci 0x1100 "Qemu virtual machine"
              Revision: 0x03
              Driver: "piix4_smbus"
              Driver Modules: "i2c_piix4"
              IRQ: 9 (no events)
              Module Alias: "pci:v00008086d00007113sv00001AF4sd00001100bc06sc80i00"
              Driver Info #0:
                Driver Status: i2c_piix4 is active
                Driver Activation Cmd: "modprobe i2c_piix4"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "13": {
                    "PCI 01.0": "0601 ISA bridge",
                    "Note": "Created at pci.386",
                    "Unique ID": "vSkL.ucdhKwLeeAA",
                    "SysFS ID": "/devices/pci0000:00/0000:00:01.0",
                    "SysFS BusID": "0000:00:01.0",
                    "Hardware Class": "bridge",
                    "Model": "Red Hat Qemu virtual machine",
                    "Vendor": 'pci 0x8086 "Intel Corporation"',
                    "Device": 'pci 0x7000 "82371SB PIIX3 ISA [Natoma/Triton II]"',
                    "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
                    "Module Alias": (
                        "pci:v00008086d00007000sv00001AF4sd00001100bc06sc01i00"
                    ),
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "14": {
                    "PCI 00.0": "0600 Host bridge",
                    "Note": "Created at pci.386",
                    "Unique ID": "qLht.YeL3TKDjrxE",
                    "SysFS ID": "/devices/pci0000:00/0000:00:00.0",
                    "SysFS BusID": "0000:00:00.0",
                    "Hardware Class": "bridge",
                    "Model": "Red Hat Qemu virtual machine",
                    "Vendor": 'pci 0x8086 "Intel Corporation"',
                    "Device": 'pci 0x1237 "440FX - 82441FX PMC [Natoma]"',
                    "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
                    "Revision": "0x02",
                    "Module Alias": (
                        "pci:v00008086d00001237sv00001AF4sd00001100bc06sc00i00"
                    ),
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "15": {
                    "PCI 01.3": "0680 Bridge",
                    "Note": "Created at pci.386",
                    "Unique ID": "VRCs.M9Cc8lcQjE2",
                    "SysFS ID": "/devices/pci0000:00/0000:00:01.3",
                    "SysFS BusID": "0000:00:01.3",
                    "Hardware Class": "bridge",
                    "Model": "Red Hat Qemu virtual machine",
                    "Vendor": 'pci 0x8086 "Intel Corporation"',
                    "Device": 'pci 0x7113 "82371AB/EB/MB PIIX4 ACPI"',
                    "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
                    "Revision": "0x03",
                    "Driver": ["piix4_smbus"],
                    "Driver Modules": ["i2c_piix4"],
                    "IRQ": "9 (no events)",
                    "Module Alias": (
                        "pci:v00008086d00007113sv00001AF4sd00001100bc06sc80i00"
                    ),
                    "Driver Info #0": {
                        "Driver Status": "i2c_piix4 is active",
                        "Driver Activation Cmd": "modprobe i2c_piix4",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_ethernet(self):
        hwinfo = textwrap.dedent(
            """
            16: PCI 03.0: 0200 Ethernet controller
              [Created at pci.386]
              Unique ID: 3hqH.pkM7KXDR457
              SysFS ID: /devices/pci0000:00/0000:00:03.0
              SysFS BusID: 0000:00:03.0
              Hardware Class: unknown
              Model: "Red Hat Virtio network device"
              Vendor: pci 0x1af4 "Red Hat, Inc."
              Device: pci 0x1000 "Virtio network device"
              SubVendor: pci 0x1af4 "Red Hat, Inc."
              SubDevice: pci 0x0001
              Driver: "virtio-pci"
              Driver Modules: "virtio_pci"
              I/O Ports: 0xc000-0xc01f (rw)
              Memory Range: 0xfebd1000-0xfebd1fff (rw,non-prefetchable)
              Memory Range: 0xfe000000-0xfe003fff (ro,non-prefetchable)
              Memory Range: 0xfeb80000-0xfebbffff (ro,non-prefetchable,disabled)
              IRQ: 11 (no events)
              Module Alias: "pci:v00001AF4d00001000sv00001AF4sd00000001bc02sc00i00"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "16": {
                    "PCI 03.0": "0200 Ethernet controller",
                    "Note": "Created at pci.386",
                    "Unique ID": "3hqH.pkM7KXDR457",
                    "SysFS ID": "/devices/pci0000:00/0000:00:03.0",
                    "SysFS BusID": "0000:00:03.0",
                    "Hardware Class": "unknown",
                    "Model": "Red Hat Virtio network device",
                    "Vendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "Device": 'pci 0x1000 "Virtio network device"',
                    "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "SubDevice": "pci 0x0001",
                    "Driver": ["virtio-pci"],
                    "Driver Modules": ["virtio_pci"],
                    "I/O Ports": "0xc000-0xc01f (rw)",
                    "Memory Range": [
                        "0xfebd1000-0xfebd1fff (rw,non-prefetchable)",
                        "0xfe000000-0xfe003fff (ro,non-prefetchable)",
                        "0xfeb80000-0xfebbffff (ro,non-prefetchable,disabled)",
                    ],
                    "IRQ": "11 (no events)",
                    "Module Alias": (
                        "pci:v00001AF4d00001000sv00001AF4sd00000001bc02sc00i00"
                    ),
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_storage(self):
        hwinfo = textwrap.dedent(
            """
            17: PCI 01.1: 0101 IDE interface (ISA Compatibility mode-only controller, supports bus mts bus mastering)
              [Created at pci.386]
              Unique ID: mnDB.3sKqaxiizg6
              SysFS ID: /devices/pci0000:00/0000:00:01.1
              SysFS BusID: 0000:00:01.1
              Hardware Class: storage
              Model: "Red Hat Qemu virtual machine"
              Vendor: pci 0x8086 "Intel Corporation"
              Device: pci 0x7010 "82371SB PIIX3 IDE [Natoma/Triton II]"
              SubVendor: pci 0x1af4 "Red Hat, Inc."
              SubDevice: pci 0x1100 "Qemu virtual machine"
              Driver: "ata_piix"
              Driver Modules: "ata_piix"
              I/O Ports: 0x1f0-0x1f7 (rw)
              I/O Port: 0x3f6 (rw)
              I/O Ports: 0x170-0x177 (rw)
              I/O Port: 0x376 (rw)
              I/O Ports: 0xc020-0xc02f (rw)
              Module Alias: "pci:v00008086d00007010sv00001AF4sd00001100bc01sc01i80"
              Driver Info #0:
                Driver Status: ata_piix is active
                Driver Activation Cmd: "modprobe ata_piix"
              Driver Info #1:
                Driver Status: ata_generic is active
                Driver Activation Cmd: "modprobe ata_generic"
              Driver Info #2:
                Driver Status: pata_acpi is active
                Driver Activation Cmd: "modprobe pata_acpi"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "17": {
                    "PCI 01.1": (
                        "0101 IDE interface (ISA Compatibility mode-only controller,"
                        " supports bus mts bus mastering)"
                    ),
                    "Note": "Created at pci.386",
                    "Unique ID": "mnDB.3sKqaxiizg6",
                    "SysFS ID": "/devices/pci0000:00/0000:00:01.1",
                    "SysFS BusID": "0000:00:01.1",
                    "Hardware Class": "storage",
                    "Model": "Red Hat Qemu virtual machine",
                    "Vendor": 'pci 0x8086 "Intel Corporation"',
                    "Device": 'pci 0x7010 "82371SB PIIX3 IDE [Natoma/Triton II]"',
                    "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
                    "Driver": ["ata_piix"],
                    "Driver Modules": ["ata_piix"],
                    "I/O Ports": [
                        "0x1f0-0x1f7 (rw)",
                        "0x3f6 (rw)",
                        "0x170-0x177 (rw)",
                        "0x376 (rw)",
                        "0xc020-0xc02f (rw)",
                    ],
                    "Module Alias": (
                        "pci:v00008086d00007010sv00001AF4sd00001100bc01sc01i80"
                    ),
                    "Driver Info #0": {
                        "Driver Status": "ata_piix is active",
                        "Driver Activation Cmd": "modprobe ata_piix",
                    },
                    "Driver Info #1": {
                        "Driver Status": "ata_generic is active",
                        "Driver Activation Cmd": "modprobe ata_generic",
                    },
                    "Driver Info #2": {
                        "Driver Status": "pata_acpi is active",
                        "Driver Activation Cmd": "modprobe pata_acpi",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_video(self):
        hwinfo = textwrap.dedent(
            """
            18: PCI 02.0: 0300 VGA compatible controller (VGA)
              [Created at pci.386]
              Unique ID: _Znp.WspiKb87LiA
              SysFS ID: /devices/pci0000:00/0000:00:02.0
              SysFS BusID: 0000:00:02.0
              Hardware Class: graphics card
              Model: "VGA compatible controller"
              Vendor: pci 0x1234
              Device: pci 0x1111
              SubVendor: pci 0x1af4 "Red Hat, Inc."
              SubDevice: pci 0x1100
              Revision: 0x02
              Driver: "bochs-drm"
              Driver Modules: "bochs_drm"
              Memory Range: 0xfd000000-0xfdffffff (ro,non-prefetchable)
              Memory Range: 0xfebd0000-0xfebd0fff (rw,non-prefetchable)
              Memory Range: 0x000c0000-0x000dffff (rw,non-prefetchable,disabled)
              Module Alias: "pci:v00001234d00001111sv00001AF4sd00001100bc03sc00i00"
              Driver Info #0:
                Driver Status: bochs_drm is active
                Driver Activation Cmd: "modprobe bochs_drm"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "18": {
                    "PCI 02.0": "0300 VGA compatible controller (VGA)",
                    "Note": "Created at pci.386",
                    "Unique ID": "_Znp.WspiKb87LiA",
                    "SysFS ID": "/devices/pci0000:00/0000:00:02.0",
                    "SysFS BusID": "0000:00:02.0",
                    "Hardware Class": "graphics card",
                    "Model": "VGA compatible controller",
                    "Vendor": "pci 0x1234",
                    "Device": "pci 0x1111",
                    "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
                    "SubDevice": "pci 0x1100",
                    "Revision": "0x02",
                    "Driver": ["bochs-drm"],
                    "Driver Modules": ["bochs_drm"],
                    "Memory Range": [
                        "0xfd000000-0xfdffffff (ro,non-prefetchable)",
                        "0xfebd0000-0xfebd0fff (rw,non-prefetchable)",
                        "0x000c0000-0x000dffff (rw,non-prefetchable,disabled)",
                    ],
                    "Module Alias": (
                        "pci:v00001234d00001111sv00001AF4sd00001100bc03sc00i00"
                    ),
                    "Driver Info #0": {
                        "Driver Status": "bochs_drm is active",
                        "Driver Activation Cmd": "modprobe bochs_drm",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_network(self):
        hwinfo = textwrap.dedent(
            """
            19: Virtio 00.0: 0200 Ethernet controller
              [Created at pci.1679]
              Unique ID: vWuh.VIRhsc57kTD
              Parent ID: 3hqH.pkM7KXDR457
              SysFS ID: /devices/pci0000:00/0000:00:03.0/virtio0
              SysFS BusID: virtio0
              Hardware Class: network
              Model: "Virtio Ethernet Card 0"
              Vendor: int 0x6014 "Virtio"
              Device: int 0x0001 "Ethernet Card 0"
              Driver: "virtio_net"
              Driver Modules: "virtio_net"
              Device File: ens3
              HW Address: 52:54:00:12:34:56
              Permanent HW Address: 52:54:00:12:34:56
              Link detected: yes
              Module Alias: "virtio:d00000001v00001AF4"
              Driver Info #0:
                Driver Status: virtio_net is active
                Driver Activation Cmd: "modprobe virtio_net"
              Config Status: cfg=new, avail=yes, need=no, active=unknown
              Attached to: #16 (Ethernet controller)

            20: None 00.0: 0700 Serial controller (16550)
              [Created at serial.74]
              Unique ID: S_Uw.3fyvFV+mbWD
              Hardware Class: unknown
              Model: "16550A"
              Device: "16550A"
              Device File: /dev/ttyS0
              Tags: mouse, modem, braille
              I/O Ports: 0x3f8-0x3ff (rw)
              IRQ: 4 (55234 events)
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "19": {
                    "Virtio 00.0": "0200 Ethernet controller",
                    "Note": "Created at pci.1679",
                    "Unique ID": "vWuh.VIRhsc57kTD",
                    "Parent ID": "3hqH.pkM7KXDR457",
                    "SysFS ID": "/devices/pci0000:00/0000:00:03.0/virtio0",
                    "SysFS BusID": "virtio0",
                    "Hardware Class": "network",
                    "Model": "Virtio Ethernet Card 0",
                    "Vendor": 'int 0x6014 "Virtio"',
                    "Device": 'int 0x0001 "Ethernet Card 0"',
                    "Driver": ["virtio_net"],
                    "Driver Modules": ["virtio_net"],
                    "Device File": "ens3",
                    "HW Address": "52:54:00:12:34:56",
                    "Permanent HW Address": "52:54:00:12:34:56",
                    "Link detected": "yes",
                    "Module Alias": "virtio:d00000001v00001AF4",
                    "Driver Info #0": {
                        "Driver Status": "virtio_net is active",
                        "Driver Activation Cmd": "modprobe virtio_net",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                    "Attached to": {"Handle": "#16 (Ethernet controller)"},
                },
                "20": {
                    "None 00.0": "0700 Serial controller (16550)",
                    "Note": "Created at serial.74",
                    "Unique ID": "S_Uw.3fyvFV+mbWD",
                    "Hardware Class": "unknown",
                    "Model": "16550A",
                    "Device": "16550A",
                    "Device File": "/dev/ttyS0",
                    "Tags": ["mouse", "modem", "braille"],
                    "I/O Ports": "0x3f8-0x3ff (rw)",
                    "IRQ": "4 (55234 events)",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_disk(self):
        hwinfo = textwrap.dedent(
            """
            21: SCSI 100.0: 10602 CD-ROM (DVD)
              [Created at block.249]
              Unique ID: KD9E.53N0UD4ozwD
              Parent ID: mnDB.3sKqaxiizg6
              SysFS ID: /class/block/sr0
              SysFS BusID: 1:0:0:0
              SysFS Device Link: /devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0
              Hardware Class: cdrom
              Model: "QEMU DVD-ROM"
              Vendor: "QEMU"
              Device: "QEMU DVD-ROM"
              Revision: "2.5+"
              Driver: "ata_piix", "sr"
              Driver Modules: "ata_piix", "sr_mod"
              Device File: /dev/sr0 (/dev/sg1)
              Device Files: /dev/sr0, /dev/cdrom, /dev/dvd, /dev/disk/by-path/pci-0000:00:01.1-ata-2, /dev/disk/by-id/ata-QEMU_DVD-ROM_QM00003, /dev/disk/by-uuid/2019-08-11-11-44-39-00, /dev/disk/by-label/CDROM
              Device Number: block 11:0 (char 21:1)
              Features: DVD, MRW, MRW-W
              Config Status: cfg=new, avail=yes, need=no, active=unknown
              Attached to: #17 (IDE interface)
              Drive Speed: 4
              Volume ID: "CDROM"
              Application: "0X5228779D"
              Publisher: "SUSE LINUX GMBH"
              Preparer: "KIWI - HTTPS://GITHUB.COM/OSINSIDE/KIWI"
              Creation date: "2019081111443900"
              El Torito info: platform 0, bootable
                Boot Catalog: at sector 0x00fa
                Media: none starting at sector 0x00fb
                Load: 2048 bytes

            22: None 00.0: 10600 Disk
              [Created at block.245]
              Unique ID: kwWm.Fxp0d3BezAE
              SysFS ID: /class/block/fd0
              SysFS BusID: floppy.0
              SysFS Device Link: /devices/platform/floppy.0
              Hardware Class: disk
              Model: "Disk"
              Driver: "floppy"
              Driver Modules: "floppy"
              Device File: /dev/fd0
              Device Number: block 2:0
              Size: 8 sectors a 512 bytes
              Capacity: 0 GB (4096 bytes)
              Drive status: no medium
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            23: IDE 00.0: 10600 Disk
              [Created at block.245]
              Unique ID: 3OOL.W8iGvCekDp8
              Parent ID: mnDB.3sKqaxiizg6
              SysFS ID: /class/block/sda
              SysFS BusID: 0:0:0:0
              SysFS Device Link: /devices/pci0000:00/0000:00:01.1/ata1/host0/target0:0:0/0:0:0:0
              Hardware Class: disk
              Model: "QEMU HARDDISK"
              Vendor: "QEMU"
              Device: "HARDDISK"
              Revision: "2.5+"
              Serial ID: "QM00001"
              Driver: "ata_piix", "sd"
              Driver Modules: "ata_piix"
              Device File: /dev/sda
              Device Files: /dev/sda, /dev/disk/by-path/pci-0000:00:01.1-ata-1, /dev/disk/by-id/ata-QEMU_HARDDISK_QM00001
              Device Number: block 8:0-8:15
              Geometry (Logical): CHS 3133/255/63
              Size: 50331648 sectors a 512 bytes
              Capacity: 24 GB (25769803776 bytes)
              Config Status: cfg=new, avail=yes, need=no, active=unknown
              Attached to: #17 (IDE interface)
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "21": {
                    "SCSI 100.0": "10602 CD-ROM (DVD)",
                    "Note": "Created at block.249",
                    "Unique ID": "KD9E.53N0UD4ozwD",
                    "Parent ID": "mnDB.3sKqaxiizg6",
                    "SysFS ID": "/class/block/sr0",
                    "SysFS BusID": "1:0:0:0",
                    "SysFS Device Link": "/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0",
                    "Hardware Class": "cdrom",
                    "Model": "QEMU DVD-ROM",
                    "Vendor": "QEMU",
                    "Device": "QEMU DVD-ROM",
                    "Revision": "2.5+",
                    "Driver": ["ata_piix", "sr"],
                    "Driver Modules": ["ata_piix", "sr_mod"],
                    "Device File": "/dev/sr0 (/dev/sg1)",
                    "Device Files": [
                        "/dev/sr0",
                        "/dev/cdrom",
                        "/dev/dvd",
                        "/dev/disk/by-path/pci-0000:00:01.1-ata-2",
                        "/dev/disk/by-id/ata-QEMU_DVD-ROM_QM00003",
                        "/dev/disk/by-uuid/2019-08-11-11-44-39-00",
                        "/dev/disk/by-label/CDROM",
                    ],
                    "Device Number": "block 11:0 (char 21:1)",
                    "Features": ["DVD", "MRW", "MRW-W"],
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                    "Attached to": {"Handle": "#17 (IDE interface)"},
                    "Drive Speed": "4",
                    "Volume ID": "CDROM",
                    "Application": "0X5228779D",
                    "Publisher": "SUSE LINUX GMBH",
                    "Preparer": "KIWI - HTTPS://GITHUB.COM/OSINSIDE/KIWI",
                    "Creation date": "2019081111443900",
                    "El Torito info": {
                        "platform": "0",
                        "bootable": "yes",
                        "Boot Catalog": "at sector 0x00fa",
                        "Media": "none starting at sector 0x00fb",
                        "Load": "2048 bytes",
                    },
                },
                "22": {
                    "None 00.0": "10600 Disk",
                    "Note": "Created at block.245",
                    "Unique ID": "kwWm.Fxp0d3BezAE",
                    "SysFS ID": "/class/block/fd0",
                    "SysFS BusID": "floppy.0",
                    "SysFS Device Link": "/devices/platform/floppy.0",
                    "Hardware Class": "disk",
                    "Model": "Disk",
                    "Driver": ["floppy"],
                    "Driver Modules": ["floppy"],
                    "Device File": "/dev/fd0",
                    "Device Number": "block 2:0",
                    "Size": "8 sectors a 512 bytes",
                    "Capacity": "0 GB (4096 bytes)",
                    "Drive status": "no medium",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "23": {
                    "IDE 00.0": "10600 Disk",
                    "Note": "Created at block.245",
                    "Unique ID": "3OOL.W8iGvCekDp8",
                    "Parent ID": "mnDB.3sKqaxiizg6",
                    "SysFS ID": "/class/block/sda",
                    "SysFS BusID": "0:0:0:0",
                    "SysFS Device Link": "/devices/pci0000:00/0000:00:01.1/ata1/host0/target0:0:0/0:0:0:0",
                    "Hardware Class": "disk",
                    "Model": "QEMU HARDDISK",
                    "Vendor": "QEMU",
                    "Device": "HARDDISK",
                    "Revision": "2.5+",
                    "Serial ID": "QM00001",
                    "Driver": ["ata_piix", "sd"],
                    "Driver Modules": ["ata_piix"],
                    "Device File": "/dev/sda",
                    "Device Files": [
                        "/dev/sda",
                        "/dev/disk/by-path/pci-0000:00:01.1-ata-1",
                        "/dev/disk/by-id/ata-QEMU_HARDDISK_QM00001",
                    ],
                    "Device Number": "block 8:0-8:15",
                    "Geometry (Logical)": "CHS 3133/255/63",
                    "Size": "50331648 sectors a 512 bytes",
                    "Capacity": "24 GB (25769803776 bytes)",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                    "Attached to": {"Handle": "#17 (IDE interface)"},
                },
            },
        )

    def test__hwinfo_parse_full_keyboard(self):
        hwinfo = textwrap.dedent(
            """
            24: PS/2 00.0: 10800 Keyboard
              [Created at input.226]
              Unique ID: nLyy.+49ps10DtUF
              Hardware Class: keyboard
              Model: "AT Translated Set 2 keyboard"
              Vendor: 0x0001
              Device: 0x0001 "AT Translated Set 2 keyboard"
              Compatible to: int 0x0211 0x0001
              Device File: /dev/input/event0
              Device Files: /dev/input/event0, /dev/input/by-path/platform-i8042-serio-0-event-kbd
              Device Number: char 13:64
              Driver Info #0:
                XkbRules: xfree86
                XkbModel: pc104
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "24": {
                    "PS/2 00.0": "10800 Keyboard",
                    "Note": "Created at input.226",
                    "Unique ID": "nLyy.+49ps10DtUF",
                    "Hardware Class": "keyboard",
                    "Model": "AT Translated Set 2 keyboard",
                    "Vendor": "0x0001",
                    "Device": '0x0001 "AT Translated Set 2 keyboard"',
                    "Compatible to": "int 0x0211 0x0001",
                    "Device File": "/dev/input/event0",
                    "Device Files": [
                        "/dev/input/event0",
                        "/dev/input/by-path/platform-i8042-serio-0-event-kbd",
                    ],
                    "Device Number": "char 13:64",
                    "Driver Info #0": {"XkbRules": "xfree86", "XkbModel": "pc104"},
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_mouse(self):
        hwinfo = textwrap.dedent(
            """
            25: PS/2 00.0: 10500 PS/2 Mouse
              [Created at input.249]
              Unique ID: AH6Q.mYF0pYoTCW7
              Hardware Class: mouse
              Model: "VirtualPS/2 VMware VMMouse"
              Vendor: 0x0002
              Device: 0x0013 "VirtualPS/2 VMware VMMouse"
              Compatible to: int 0x0210 0x0003
              Device File: /dev/input/mice (/dev/input/mouse0)
              Device Files: /dev/input/mice, /dev/input/mouse0, /dev/input/event1, /dev/input/by-path/platform-i8042-serio-1-event-mouse, /dev/input/by-path/platform-i8042-serio-1-mouse
              Device Number: char 13:63 (char 13:32)
              Driver Info #0:
                Buttons: 3
                Wheels: 0
                XFree86 Protocol: explorerps/2
                GPM Protocol: exps2
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            26: PS/2 00.0: 10500 PS/2 Mouse
              [Created at input.249]
              Unique ID: AH6Q.++hSeDccb2F
              Hardware Class: mouse
              Model: "VirtualPS/2 VMware VMMouse"
              Vendor: 0x0002
              Device: 0x0013 "VirtualPS/2 VMware VMMouse"
              Compatible to: int 0x0210 0x0012
              Device File: /dev/input/mice (/dev/input/mouse1)
              Device Files: /dev/input/mice, /dev/input/mouse1, /dev/input/event2
              Device Number: char 13:63 (char 13:33)
              Driver Info #0:
                Buttons: 2
                Wheels: 1
                XFree86 Protocol: explorerps/2
                GPM Protocol: exps2
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "25": {
                    "PS/2 00.0": "10500 PS/2 Mouse",
                    "Note": "Created at input.249",
                    "Unique ID": "AH6Q.mYF0pYoTCW7",
                    "Hardware Class": "mouse",
                    "Model": "VirtualPS/2 VMware VMMouse",
                    "Vendor": "0x0002",
                    "Device": '0x0013 "VirtualPS/2 VMware VMMouse"',
                    "Compatible to": "int 0x0210 0x0003",
                    "Device File": "/dev/input/mice (/dev/input/mouse0)",
                    "Device Files": [
                        "/dev/input/mice",
                        "/dev/input/mouse0",
                        "/dev/input/event1",
                        "/dev/input/by-path/platform-i8042-serio-1-event-mouse",
                        "/dev/input/by-path/platform-i8042-serio-1-mouse",
                    ],
                    "Device Number": "char 13:63 (char 13:32)",
                    "Driver Info #0": {
                        "Buttons": "3",
                        "Wheels": "0",
                        "XFree86 Protocol": "explorerps/2",
                        "GPM Protocol": "exps2",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "26": {
                    "PS/2 00.0": "10500 PS/2 Mouse",
                    "Note": "Created at input.249",
                    "Unique ID": "AH6Q.++hSeDccb2F",
                    "Hardware Class": "mouse",
                    "Model": "VirtualPS/2 VMware VMMouse",
                    "Vendor": "0x0002",
                    "Device": '0x0013 "VirtualPS/2 VMware VMMouse"',
                    "Compatible to": "int 0x0210 0x0012",
                    "Device File": "/dev/input/mice (/dev/input/mouse1)",
                    "Device Files": [
                        "/dev/input/mice",
                        "/dev/input/mouse1",
                        "/dev/input/event2",
                    ],
                    "Device Number": "char 13:63 (char 13:33)",
                    "Driver Info #0": {
                        "Buttons": "2",
                        "Wheels": "1",
                        "XFree86 Protocol": "explorerps/2",
                        "GPM Protocol": "exps2",
                    },
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_cpu(self):
        hwinfo = textwrap.dedent(
            """
            27: None 00.0: 10103 CPU
              [Created at cpu.462]
              Unique ID: rdCR.j8NaKXDZtZ6
              Hardware Class: cpu
              Arch: X86-64
              Vendor: "GenuineIntel"
              Model: 6.6.3 "QEMU Virtual CPU version 2.5+"
              Features: fpu,de,pse,tsc,msr,pae,mce,cx8,apic,sep,mtrr,pge,mca,cmov,pse36,clflush,mmx,fxsr,sse,sse2,syscall,nx,lm,rep_good,nopl,xtopology,cpuid,tsc_known_freq,pni,cx16,x2apic,hypervisor,lahf_lm,cpuid_fault,pti
              Clock: 3591 MHz
              BogoMips: 7182.68
              Cache: 16384 kb
              Config Status: cfg=new, avail=yes, need=no, active=unknown
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "27": {
                    "None 00.0": "10103 CPU",
                    "Note": "Created at cpu.462",
                    "Unique ID": "rdCR.j8NaKXDZtZ6",
                    "Hardware Class": "cpu",
                    "Arch": "X86-64",
                    "Vendor": "GenuineIntel",
                    "Model": '6.6.3 "QEMU Virtual CPU version 2.5+"',
                    "Features": [
                        "fpu",
                        "de",
                        "pse",
                        "tsc",
                        "msr",
                        "pae",
                        "mce",
                        "cx8",
                        "apic",
                        "sep",
                        "mtrr",
                        "pge",
                        "mca",
                        "cmov",
                        "pse36",
                        "clflush",
                        "mmx",
                        "fxsr",
                        "sse",
                        "sse2",
                        "syscall",
                        "nx",
                        "lm",
                        "rep_good",
                        "nopl",
                        "xtopology",
                        "cpuid",
                        "tsc_known_freq",
                        "pni",
                        "cx16",
                        "x2apic",
                        "hypervisor",
                        "lahf_lm",
                        "cpuid_fault",
                        "pti",
                    ],
                    "Clock": "3591 MHz",
                    "BogoMips": "7182.68",
                    "Cache": "16384 kb",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
            },
        )

    def test__hwinfo_parse_full_nic(self):
        hwinfo = textwrap.dedent(
            """
            28: None 00.0: 10700 Loopback
              [Created at net.126]
              Unique ID: ZsBS.GQNx7L4uPNA
              SysFS ID: /class/net/lo
              Hardware Class: network interface
              Model: "Loopback network interface"
              Device File: lo
              Link detected: yes
              Config Status: cfg=new, avail=yes, need=no, active=unknown

            29: None 03.0: 10701 Ethernet
              [Created at net.126]
              Unique ID: U2Mp.ndpeucax6V1
              Parent ID: vWuh.VIRhsc57kTD
              SysFS ID: /class/net/ens3
              SysFS Device Link: /devices/pci0000:00/0000:00:03.0/virtio0
              Hardware Class: network interface
              Model: "Ethernet network interface"
              Driver: "virtio_net"
              Driver Modules: "virtio_net"
              Device File: ens3
              HW Address: 52:54:00:12:34:56
              Permanent HW Address: 52:54:00:12:34:56
              Link detected: yes
              Config Status: cfg=new, avail=yes, need=no, active=unknown
              Attached to: #19 (Ethernet controller)
        """
        )
        self.assertEqual(
            devinfo._hwinfo_parse_full(hwinfo),
            {
                "28": {
                    "None 00.0": "10700 Loopback",
                    "Note": "Created at net.126",
                    "Unique ID": "ZsBS.GQNx7L4uPNA",
                    "SysFS ID": "/class/net/lo",
                    "Hardware Class": "network interface",
                    "Model": "Loopback network interface",
                    "Device File": "lo",
                    "Link detected": "yes",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                },
                "29": {
                    "None 03.0": "10701 Ethernet",
                    "Note": "Created at net.126",
                    "Unique ID": "U2Mp.ndpeucax6V1",
                    "Parent ID": "vWuh.VIRhsc57kTD",
                    "SysFS ID": "/class/net/ens3",
                    "SysFS Device Link": "/devices/pci0000:00/0000:00:03.0/virtio0",
                    "Hardware Class": "network interface",
                    "Model": "Ethernet network interface",
                    "Driver": ["virtio_net"],
                    "Driver Modules": ["virtio_net"],
                    "Device File": "ens3",
                    "HW Address": "52:54:00:12:34:56",
                    "Permanent HW Address": "52:54:00:12:34:56",
                    "Link detected": "yes",
                    "Config Status": {
                        "cfg": "new",
                        "avail": "yes",
                        "need": "no",
                        "active": "unknown",
                    },
                    "Attached to": {"Handle": "#19 (Ethernet controller)"},
                },
            },
        )



"""
Return data to a memcache server

To enable this returner the minion will need the python client for memcache
installed and the following values configured in the minion or master
config, these are the defaults.

.. code-block:: yaml

    memcache.host: 'localhost'
    memcache.port: '11211'

Alternative configuration values can be used by prefacing the configuration.
Any values not found in the alternative configuration will be pulled from
the default location.

.. code-block:: yaml

    alternative.memcache.host: 'localhost'
    alternative.memcache.port: '11211'

python2-memcache uses 'localhost' and '11211' as syntax on connection.

To use the memcache returner, append '--return memcache' to the salt command.

.. code-block:: bash

    salt '*' test.ping --return memcache

To use the alternative configuration, append '--return_config alternative' to the salt command.

.. versionadded:: 2015.5.0

.. code-block:: bash

    salt '*' test.ping --return memcache --return_config alternative

To override individual configuration items, append --return_kwargs '{"key:": "value"}' to the salt command.

.. versionadded:: 2016.3.0

.. code-block:: bash

    salt '*' test.ping --return memcache --return_kwargs '{"host": "hostname.domain.com"}'

"""

import logging

import salt.returners
import salt.utils.jid
import salt.utils.json

log = logging.getLogger(__name__)

try:
    import memcache

    HAS_MEMCACHE = True
except ImportError:
    HAS_MEMCACHE = False

# Define the module's virtual name
__virtualname__ = "memcache"


def __virtual__():
    if not HAS_MEMCACHE:
        return (
            False,
            "Could not import memcache returner; "
            "memcache python client is not installed.",
        )
    return __virtualname__


def _get_options(ret=None):
    """
    Get the memcache options from salt.
    """
    attrs = {"host": "host", "port": "port"}

    _options = salt.returners.get_returner_options(
        __virtualname__, ret, attrs, __salt__=__salt__, __opts__=__opts__
    )
    return _options


def _get_serv(ret):
    """
    Return a memcache server object
    """

    _options = _get_options(ret)
    host = _options.get("host")
    port = _options.get("port")

    log.debug("memcache server: %s:%s", host, port)
    if not host or not port:
        log.error("Host or port not defined in salt config")
        return

    # Combine host and port to conform syntax of python memcache client
    memcacheoptions = (host, port)

    return memcache.Client(["{}:{}".format(*memcacheoptions)], debug=0)
    # # TODO: make memcacheoptions cluster aware
    # Servers can be passed in two forms:
    # 1. Strings of the form C{"host:port"}, which implies a default weight of 1
    # 2. Tuples of the form C{("host:port", weight)}, where C{weight} is
    #    an integer weight value.


def _get_list(serv, key):
    value = serv.get(key)
    if value:
        return value.strip(",").split(",")
    return []


def _append_list(serv, key, value):
    if value in _get_list(serv, key):
        return
    r = serv.append(key, "{},".format(value))
    if not r:
        serv.add(key, "{},".format(value))


def prep_jid(nocache=False, passed_jid=None):  # pylint: disable=unused-argument
    """
    Do any work necessary to prepare a JID, including sending a custom id
    """
    return passed_jid if passed_jid is not None else salt.utils.jid.gen_jid(__opts__)


def returner(ret):
    """
    Return data to a memcache data store
    """
    serv = _get_serv(ret)
    minion = ret["id"]
    jid = ret["jid"]
    fun = ret["fun"]
    rets = salt.utils.json.dumps(ret)
    serv.set("{}:{}".format(jid, minion), rets)  # cache for get_jid
    serv.set("{}:{}".format(fun, minion), rets)  # cache for get_fun

    # The following operations are neither efficient nor atomic.
    # If there is a way to make them so, this should be updated.
    _append_list(serv, "minions", minion)
    _append_list(serv, "jids", jid)


def save_load(jid, load, minions=None):
    """
    Save the load to the specified jid
    """
    serv = _get_serv(ret=None)
    serv.set(jid, salt.utils.json.dumps(load))
    _append_list(serv, "jids", jid)


def save_minions(jid, minions, syndic_id=None):  # pylint: disable=unused-argument
    """
    Included for API consistency
    """


def get_load(jid):
    """
    Return the load data that marks a specified jid
    """
    serv = _get_serv(ret=None)
    data = serv.get(jid)
    if data:
        return salt.utils.json.loads(data)
    return {}


def get_jid(jid):
    """
    Return the information returned when the specified job id was executed
    """
    serv = _get_serv(ret=None)
    minions = _get_list(serv, "minions")
    returns = serv.get_multi(minions, key_prefix="{}:".format(jid))
    # returns = {minion: return, minion: return, ...}
    ret = {}
    for minion, data in returns.items():
        ret[minion] = salt.utils.json.loads(data)
    return ret


def get_fun(fun):
    """
    Return a dict of the last function called for all minions
    """
    serv = _get_serv(ret=None)
    minions = _get_list(serv, "minions")
    returns = serv.get_multi(minions, key_prefix="{}:".format(fun))
    # returns = {minion: return, minion: return, ...}
    ret = {}
    for minion, data in returns.items():
        ret[minion] = salt.utils.json.loads(data)
    return ret


def get_jids():
    """
    Return a list of all job ids
    """
    serv = _get_serv(ret=None)
    jids = _get_list(serv, "jids")
    loads = serv.get_multi(jids)  # {jid: load, jid: load, ...}
    ret = {}
    for jid, load in loads.items():
        ret[jid] = salt.utils.jid.format_jid_instance(jid, salt.utils.json.loads(load))
    return ret


def get_minions():
    """
    Return a list of minions
    """
    serv = _get_serv(ret=None)
    return _get_list(serv, "minions")



"""
Support for Alternatives system

:codeauthor: Radek Rada <radek.rada@gmail.com>
"""

import logging
import os

import salt.utils.files
import salt.utils.path

__outputter__ = {
    "display": "txt",
    "install": "txt",
    "remove": "txt",
}

log = logging.getLogger(__name__)

# Don't shadow built-in's.
__func_alias__ = {"set_": "set"}


def __virtual__():
    """
    Only if alternatives dir is available
    """
    if os.path.isdir("/etc/alternatives"):
        return True
    return (False, "Cannot load alternatives module: /etc/alternatives dir not found")


def _get_cmd():
    """
    Alteratives commands and differ across distributions
    """
    if __grains__["os_family"] == "RedHat":
        return "alternatives"
    return "update-alternatives"


def display(name):
    """
    Display alternatives settings for defined command name

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.display editor
    """
    cmd = [_get_cmd(), "--display", name]
    out = __salt__["cmd.run_all"](cmd, python_shell=False, ignore_retcode=True)
    if out["retcode"] > 0 and out["stderr"] != "":
        return out["stderr"]
    return out["stdout"]


def show_link(name):
    """
    Display master link for the alternative

    .. versionadded:: 2015.8.13,2016.3.4,2016.11.0

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.show_link editor
    """

    if __grains__["os_family"] == "RedHat":
        path = "/var/lib/"
    elif __grains__["os_family"] == "Suse":
        path = "/var/lib/rpm/"
    else:
        path = "/var/lib/dpkg/"

    path += "alternatives/{}".format(name)

    try:
        with salt.utils.files.fopen(path, "rb") as r_file:
            contents = salt.utils.stringutils.to_unicode(r_file.read())
            return contents.splitlines(True)[1].rstrip("\n")
    except OSError:
        log.error("alternatives: %s does not exist", name)
    except (OSError, IndexError) as exc:  # pylint: disable=duplicate-except
        log.error(
            "alternatives: unable to get master link for %s. Exception: %s",
            name,
            exc,
        )

    return False


def show_current(name):
    """
    Display the current highest-priority alternative for a given alternatives
    link

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.show_current editor
    """
    try:
        return _read_link(name)
    except OSError:
        log.error("alternative: %s does not exist", name)
    return False


def check_exists(name, path):
    """
    Check if the given path is an alternative for a name.

    .. versionadded:: 2015.8.4

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.check_exists name path
    """
    cmd = [_get_cmd(), "--display", name]
    out = __salt__["cmd.run_all"](cmd, python_shell=False, ignore_retcode=True)

    if out["retcode"] > 0 and out["stderr"] != "":
        return False

    return any(line.startswith(path) for line in out["stdout"].splitlines())


def check_installed(name, path):
    """
    Check if the current highest-priority match for a given alternatives link
    is set to the desired path

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.check_installed name path
    """
    try:
        return _read_link(name) == path
    except OSError:
        return False


def install(name, link, path, priority):
    """
    Install symbolic links determining default commands

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.install editor /usr/bin/editor /usr/bin/emacs23 50
    """
    cmd = [_get_cmd(), "--install", link, name, path, str(priority)]
    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out["retcode"] > 0 and out["stderr"] != "":
        return out["stderr"]
    return out["stdout"]


def remove(name, path):
    """
    Remove symbolic links determining the default commands.

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.remove name path
    """
    cmd = [_get_cmd(), "--remove", name, path]
    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out["retcode"] > 0:
        return out["stderr"]
    return out["stdout"]


def auto(name):
    """
    Trigger alternatives to set the path for <name> as
    specified by priority.

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.auto name
    """
    cmd = [_get_cmd(), "--auto", name]
    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out["retcode"] > 0:
        return out["stderr"]
    return out["stdout"]


def set_(name, path):
    """
    Manually set the alternative <path> for <name>.

    CLI Example:

    .. code-block:: bash

        salt '*' alternatives.set name path
    """
    cmd = [_get_cmd(), "--set", name, path]
    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out["retcode"] > 0:
        return out["stderr"]
    return out["stdout"]


def _read_link(name):
    """
    Read the link from /etc/alternatives

    Throws an OSError if the link does not exist
    """
    alt_link_path = "/etc/alternatives/{}".format(name)
    return salt.utils.path.readlink(alt_link_path)



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""

import os

import salt.modules.debconfmod as debconfmod
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DebconfmodTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.DebconfmodTestCase
    """

    def setup_loader_modules(self):
        return {debconfmod: {}}

    def test_get_selections(self):
        """
        Test for Answers to debconf questions for all packages
        """
        mock = MagicMock(return_value=[])
        with patch.dict(debconfmod.__salt__, {"cmd.run_stdout": mock}):
            with patch.object(debconfmod, "_unpack_lines", mock):
                self.assertEqual(debconfmod.get_selections(False), {})

    def test_show(self):
        """
        Test for Answers to debconf questions for a package
        """
        mock = MagicMock(return_value={})
        with patch.object(debconfmod, "get_selections", mock):
            self.assertEqual(debconfmod.show("name"), None)

    def test_set_(self):
        """
        Test for Set answers to debconf questions for a package.
        """
        mock = MagicMock(return_value=None)
        with patch.object(os, "write", mock):
            with patch.object(os, "close", mock):
                with patch.object(debconfmod, "_set_file", mock):
                    with patch.object(os, "unlink", mock):
                        self.assertTrue(
                            debconfmod.set_("package", "question", "type", "value")
                        )

    def test_set_template(self):
        """
        Test for Set answers to debconf questions from a template.
        """
        mock = MagicMock(return_value="A")
        with patch.dict(debconfmod.__salt__, {"cp.get_template": mock}):
            with patch.object(debconfmod, "set_file", mock):
                self.assertEqual(
                    debconfmod.set_template(
                        "path", "template", "context", "defaults", "saltenv"
                    ),
                    "A",
                )

    def test_set_file(self):
        """
        Test for Set answers to debconf questions from a file.
        """
        mock = MagicMock(return_value="A")
        with patch.dict(debconfmod.__salt__, {"cp.cache_file": mock}):
            mock = MagicMock(return_value=None)
            with patch.object(debconfmod, "_set_file", mock):
                self.assertTrue(debconfmod.set_file("path"))

        mock = MagicMock(return_value=False)
        with patch.dict(debconfmod.__salt__, {"cp.cache_file": mock}):
            mock = MagicMock(return_value=None)
            with patch.object(debconfmod, "_set_file", mock):
                self.assertFalse(debconfmod.set_file("path"))



"""
tests.unit.modules.test_glassfish
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Unit tests for the glassfish module
"""
import logging

import salt.modules.glassfish as glassfish
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class GlassFishTest(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {glassfish: {}}

    def test__api_get(self):
        get_mock = MagicMock()
        with patch("salt.modules.glassfish.requests.get", get_mock):
            glassfish._api_get("ThePath", server=glassfish.DEFAULT_SERVER)

        get_mock.assert_called_once_with(
            headers={
                "Accept": "application/json",
                "Content-Type": "application/json",
                "X-Requested-By": "GlassFish REST HTML interface",
            },
            url="http://localhost:4848/management/domain/ThePath",
            verify=True,
            auth=None,
        )

    def test__api_post(self):
        post_mock = MagicMock()
        with patch("salt.modules.glassfish.requests.post", post_mock):
            glassfish._api_post("ThePath", {1: 1}, server=glassfish.DEFAULT_SERVER)

        post_mock.assert_called_once_with(
            headers={
                "Accept": "application/json",
                "Content-Type": "application/json",
                "X-Requested-By": "GlassFish REST HTML interface",
            },
            url="http://localhost:4848/management/domain/ThePath",
            verify=True,
            auth=None,
            data='{"1": 1}',
        )

    def test__api_delete(self):
        delete_mock = MagicMock()
        with patch("salt.modules.glassfish.requests.delete", delete_mock):
            glassfish._api_delete("ThePath", {1: 1}, server=glassfish.DEFAULT_SERVER)

        delete_mock.assert_called_once_with(
            headers={
                "Accept": "application/json",
                "Content-Type": "application/json",
                "X-Requested-By": "GlassFish REST HTML interface",
            },
            url="http://localhost:4848/management/domain/ThePath",
            verify=True,
            auth=None,
            params={1: 1},
        )



import salt.modules.swarm
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase


class SwarmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.swarm
    """

    def setup_loader_modules(self):
        return {salt.modules.swarm: {}}

    def test___virtual___valid_docker_module(self):
        """
        Test that __virtual__ requires a proper loaded docker library
        """

        class ValidDockerModule:
            class APIClient:
                pass

            def from_env(self):
                pass

        with patch(
            "salt.modules.swarm.HAS_DOCKER",
            salt.modules.swarm._is_docker_module(ValidDockerModule()),
        ):
            self.assertEqual(
                salt.modules.swarm.__virtual__(), salt.modules.swarm.__virtualname__
            )

    def test___virtual___not_valid_docker_module(self):
        class NotValidDockerModule:
            pass

        with patch(
            "salt.modules.swarm.HAS_DOCKER",
            salt.modules.swarm._is_docker_module(NotValidDockerModule()),
        ):
            ret = salt.modules.swarm.__virtual__()
            self.assertEqual(len(ret), 2)
            self.assertFalse(ret[0])



"""
Management of Microsoft SQLServer Databases
===========================================

The mssql_database module is used to create
and manage SQL Server Databases

.. code-block:: yaml

    yolo:
      mssql_database.present
"""

import collections


def __virtual__():
    """
    Only load if the mssql module is present
    """
    if "mssql.version" in __salt__:
        return True
    return (False, "mssql module could not be loaded")


def _normalize_options(options):
    if type(options) in [dict, collections.OrderedDict]:
        return ["{}={}".format(k, v) for k, v in options.items()]
    if type(options) is list and (not options or type(options[0]) is str):
        return options
    # Invalid options
    if type(options) is not list or type(options[0]) not in [
        dict,
        collections.OrderedDict,
    ]:
        return []
    return [o for d in options for o in _normalize_options(d)]


def present(name, containment="NONE", options=None, **kwargs):
    """
    Ensure that the named database is present with the specified options

    name
        The name of the database to manage
    containment
        Defaults to NONE
    options
        Can be a list of strings, a dictionary, or a list of dictionaries
    """
    ret = {"name": name, "changes": {}, "result": True, "comment": ""}

    if __salt__["mssql.db_exists"](name, **kwargs):
        ret[
            "comment"
        ] = "Database {} is already present (Not going to try to set its options)".format(
            name
        )
        return ret
    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = "Database {} is set to be added".format(name)
        return ret

    db_created = __salt__["mssql.db_create"](
        name,
        containment=containment,
        new_database_options=_normalize_options(options),
        **kwargs
    )
    if (
        db_created is not True
    ):  # Non-empty strings are also evaluated to True, so we cannot use if not db_created:
        ret["result"] = False
        ret["comment"] += "Database {} failed to be created: {}".format(
            name, db_created
        )
        return ret
    ret["comment"] += "Database {} has been added".format(name)
    ret["changes"][name] = "Present"
    return ret


def absent(name, **kwargs):
    """
    Ensure that the named database is absent

    name
        The name of the database to remove
    """
    ret = {"name": name, "changes": {}, "result": True, "comment": ""}

    if not __salt__["mssql.db_exists"](name):
        ret["comment"] = "Database {} is not present".format(name)
        return ret
    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = "Database {} is set to be removed".format(name)
        return ret
    if __salt__["mssql.db_remove"](name, **kwargs):
        ret["comment"] = "Database {} has been removed".format(name)
        ret["changes"][name] = "Absent"
        return ret
    # else:
    ret["result"] = False
    ret["comment"] = "Database {} failed to be removed".format(name)
    return ret



"""
Tests for the salt-run command
"""

import pytest
from tests.support.case import ShellCase


@pytest.mark.windows_whitelisted
@pytest.mark.usefixtures("salt_sub_minion")
class JobsTest(ShellCase):
    """
    Test the jobs runner.
    """

    @pytest.mark.slow_test
    def test_master(self):
        """
        jobs.master
        """
        ret = self.run_run_plus("jobs.master", _output="json")
        self.assertEqual(ret["return"], [])
        self.assertEqual(ret["out"], [])

    @pytest.mark.slow_test
    def test_active(self):
        """
        jobs.active
        """
        ret = self.run_run_plus("jobs.active", _output="json")
        self.assertEqual(ret["return"], {})
        self.assertEqual(ret["out"], {})

    @pytest.mark.slow_test
    def test_lookup_jid(self):
        """
        jobs.lookup_jid
        """
        ret = self.run_run_plus("jobs.lookup_jid", "23974239742394", _output="json")
        self.assertEqual(ret["return"], {})
        self.assertEqual(ret["out"], {})

    @pytest.mark.slow_test
    def test_lookup_jid_invalid(self):
        """
        jobs.lookup_jid
        """
        ret = self.run_run_plus("jobs.lookup_jid", _output="json")
        expected = "Passed invalid arguments:"
        self.assertIn(expected, ret["return"])

    @pytest.mark.slow_test
    def test_list_jobs(self):
        """
        jobs.list_jobs
        """
        self.run_salt("minion test.echo test_list_jobs")
        ret = self.run_run_plus("jobs.list_jobs", _output="json")
        self.assertIsInstance(ret["return"], dict)
        for job in ret["return"].values():
            if job["Function"] != "test.echo":
                continue
            if job["Arguments"] != ["test_list_jobs"]:
                continue
            # We our job in the list, we're good with the test
            break
        else:
            self.fail("Did not our job from the jobs.list_jobs call")


@pytest.mark.windows_whitelisted
class LocalCacheTargetTest(ShellCase):
    """
    Test that a job stored in the local_cache has target information
    """

    @pytest.mark.slow_test
    def test_target_info(self):
        """
        This is a test case for issue #48734

        PR #43454 fixed an issue where "jobs.lookup_jid" was not working
        correctly with external job caches. However, this fix for external
        job caches broke some inner workings of job storage when using the
        local_cache.

        We need to preserve the previous behavior for the local_cache, but
        keep the new behavior for other external job caches.

        If "savefstr" is called in the local cache, the target data does not
        get written to the local_cache, and the target-type gets listed as a
        "list" type instead of "glob".

        This is a regression test for fixing the local_cache behavior.
        """
        self.run_salt("minion test.echo target_info_test")
        ret = self.run_run_plus("jobs.list_jobs", _output="json")
        for item in ret["return"].values():
            if (
                item["Function"] == "test.echo"
                and item["Arguments"][0] == "target_info_test"
            ):
                job_ret = item
        tgt = job_ret["Target"]
        tgt_type = job_ret["Target-type"]

        assert tgt != "unknown-target"
        assert tgt in ["minion", "sub_minion"]
        assert tgt_type == "glob"



"""
    :codeauthor: Mike Wiebe <@mikewiebe>
"""

# Copyright (c) 2019 Cisco and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import salt.proxy.nxos as nxos_proxy
import salt.utils.nxos as nxos_utils
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, create_autospec, patch
from tests.support.unit import TestCase
from tests.unit.modules.nxos.nxos_grains import n9k_grains
from tests.unit.modules.nxos.nxos_show_cmd_output import n9k_show_ver_list


class NxosNxapiProxyTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {nxos_proxy: {"CONNECTION": "nxapi"}}

    def test_check_virtual(self):

        """UT: nxos module:check_virtual method - return value"""

        result = nxos_proxy.__virtual__()
        self.assertIn("nxos", result)

    def test_init(self):

        """UT: nxos module:init method - nxapi proxy"""

        with patch.object(nxos_proxy, "__opts__", {"proxy": {"connection": "nxapi"}}):
            with patch("salt.proxy.nxos._init_nxapi", autospec=True) as init_nxapi:
                result = nxos_proxy.init()
                self.assertEqual(result, init_nxapi.return_value)

    def test_init_opts_none(self):

        """UT: nxos module:init method - __opts__ connection is None"""

        with patch.object(nxos_proxy, "__opts__", {"proxy": {"connection": None}}):
            with patch("salt.proxy.nxos._init_nxapi", autospec=True) as init_nxapi:
                result = nxos_proxy.init()
                self.assertEqual(result, init_nxapi.return_value)

    def test_init_bad_connection_type(self):

        """UT: nxos module:init method - bad CONNECTION type"""
        with patch.object(nxos_proxy, "__opts__", {"proxy": {"connection": "unknown"}}):
            self.assertFalse(nxos_proxy.init())

    def test_initialized(self):

        """UT: nxos module:initialized method - nxapi proxy"""

        with patch(
            "salt.proxy.nxos._initialized_nxapi", autospec=True
        ) as initialized_nxapi:
            result = nxos_proxy.initialized()
            self.assertEqual(result, initialized_nxapi.return_value)

    def test_ping(self):

        """UT: nxos module:ping method - nxapi proxy"""

        with patch("salt.proxy.nxos._ping_nxapi", autospec=True) as ping_nxapi:
            result = nxos_proxy.ping()
            self.assertEqual(result, ping_nxapi.return_value)

    def test_grains(self):

        """UT: nxos module:grains method - nxapi grains"""

        with patch(
            "salt.proxy.nxos.sendline", autospec=True, return_value=n9k_show_ver_list
        ):
            result = nxos_proxy.grains()
            self.assertEqual(result, n9k_grains)

    def test_grains_cache_set(self):

        """UT: nxos module:grains method - nxapi grains cache set"""

        with patch(
            "salt.proxy.nxos.DEVICE_DETAILS", {"grains_cache": n9k_grains["nxos"]}
        ):
            with patch(
                "salt.proxy.nxos.sendline",
                autospec=True,
                return_value=n9k_show_ver_list,
            ):
                result = nxos_proxy.grains()
                self.assertEqual(result, n9k_grains)

    def test_grains_refresh(self):

        """UT: nxos module:grains_refresh method - nxapi grains"""

        device_details = {"grains_cache": None}

        with patch("salt.proxy.nxos.DEVICE_DETAILS", device_details):
            with patch("salt.proxy.nxos.grains", autospec=True) as grains:
                result = nxos_proxy.grains_refresh()
                self.assertEqual(nxos_proxy.DEVICE_DETAILS["grains_cache"], {})
                self.assertEqual(result, grains.return_value)

    def test_sendline(self):

        """UT: nxos module:sendline method - nxapi"""

        command = "show version"

        with patch("salt.proxy.nxos._nxapi_request", autospec=True) as nxapi_request:
            result = nxos_proxy.sendline(command)
            self.assertEqual(result, nxapi_request.return_value)

    def test_proxy_config(self):

        """UT: nxos module:proxy_config method - nxapi success path"""

        commands = ["feature bgp", "router bgp 65535"]

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"save_config": False}):
            with patch(
                "salt.proxy.nxos._nxapi_request", autospec=True
            ) as nxapi_request:
                result = nxos_proxy.proxy_config(commands)
                self.assertEqual(result, [commands, nxapi_request.return_value])

    def test_proxy_config_save_config(self):

        """UT: nxos module:proxy_config method - nxapi success path"""

        commands = ["feature bgp", "router bgp 65535"]

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"save_config": None}):
            with patch(
                "salt.proxy.nxos._nxapi_request", autospec=True
            ) as nxapi_request:
                result = nxos_proxy.proxy_config(commands, save_config=True)
                self.assertEqual(result, [commands, nxapi_request.return_value])

    def test__init_nxapi(self):

        """UT: nxos module:_init_nxapi method - successful connectinon"""

        opts = {"proxy": {"arg1": None}}
        nxapi_request = create_autospec(nxos_utils.nxapi_request, return_value="data")

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {}) as device_details:
            with patch(
                "salt.proxy.nxos.__utils__", {"nxos.nxapi_request": nxapi_request}
            ):
                result = nxos_proxy._init_nxapi(opts)

                self.assertTrue(device_details["initialized"])
                self.assertTrue(device_details["up"])
                self.assertTrue(device_details["save_config"])
                self.assertTrue(result)

                nxapi_request.assert_called_with("show clock", **opts["proxy"])

    def test_bad__init_nxapi(self):
        class NXAPIException(Exception):
            pass

        nxapi_request = create_autospec(
            nxos_utils.nxapi_request, side_effect=NXAPIException
        )

        with patch("salt.proxy.nxos.__utils__", {"nxos.nxapi_request": nxapi_request}):
            with patch("salt.proxy.nxos.log", autospec=True) as log:
                with self.assertRaises(NXAPIException):
                    nxos_proxy._init_nxapi({"proxy": {"host": "HOST"}})
                log.error.assert_called()

    def test__initialized_nxapi(self):

        """UT: nxos module:_initialized_nxapi method"""

        result = nxos_proxy._initialized_nxapi()
        self.assertFalse(result)

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"initialized": True}):
            result = nxos_proxy._initialized_nxapi()
            self.assertTrue(result)

    def test__ping_nxapi(self):

        """UT: nxos module:_ping_nxapi method"""

        result = nxos_proxy._ping_nxapi()
        self.assertFalse(result)

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"up": True}):
            result = nxos_proxy._ping_nxapi()
            self.assertTrue(result)

    def test__shutdown_nxapi(self):

        """UT: nxos module:_shutdown_nxapi method"""

        opts = {"id": "value"}

        with patch("salt.proxy.nxos.log", autospec=True):
            nxos_proxy._shutdown_nxapi()
            # nothing to test

    def test__nxapi_request_ssh_return(self):

        """UT: nxos module:_nxapi_request method - CONNECTION == 'ssh'"""

        commands = "show version"

        with patch("salt.proxy.nxos.CONNECTION", "ssh"):
            result = nxos_proxy._nxapi_request(commands)
            self.assertEqual("_nxapi_request is not available for ssh proxy", result)

    def test__nxapi_request_connect(self):

        """UT: nxos module:_nxapi_request method"""

        commands = "show version"
        nxapi_request = create_autospec(nxos_utils.nxapi_request, return_value="data")

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"conn_args": {"arg1": None}}):
            with patch(
                "salt.proxy.nxos.__utils__", {"nxos.nxapi_request": nxapi_request}
            ):
                result = nxos_proxy._nxapi_request(commands)
                self.assertEqual("data", result)
                nxapi_request.assert_called_with(commands, method="cli_conf", arg1=None)


class NxosSSHProxyTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {
            nxos_proxy: {
                "__opts__": {
                    "proxy": {
                        "host": "dt-n9k5-1.cisco.com",
                        "username": "admin",
                        "password": "password",
                    }
                },
                "CONNECTION": "ssh",
            }
        }

    def test_init(self):

        """UT: nxos module:init method - ssh proxy"""

        with patch("salt.proxy.nxos._init_ssh", autospec=True) as init_ssh:
            result = nxos_proxy.init()
            self.assertEqual(result, init_ssh.return_value)

    def test_init_opts_none(self):

        """UT: nxos module:init method - __opts__ connection is None"""

        with patch("salt.proxy.nxos.__opts__", {"proxy": {"connection": None}}):
            with patch("salt.proxy.nxos._init_ssh", autospec=True) as init_ssh:
                result = nxos_proxy.init()
                self.assertEqual(result, init_ssh.return_value)

    def test_initialized(self):

        """UT: nxos module:initialized method - ssh proxy"""

        with patch(
            "salt.proxy.nxos._initialized_ssh", autospec=True
        ) as initialized_ssh:
            result = nxos_proxy.initialized()
            self.assertEqual(result, initialized_ssh.return_value)

    def test_ping(self):

        """UT: nxos module:ping method - ssh proxy"""

        with patch("salt.proxy.nxos._ping_ssh", autospec=True) as ping_ssh:
            result = nxos_proxy.ping()
            self.assertEqual(result, ping_ssh.return_value)

    def test_grains(self):

        """UT: nxos module:grains method - ssh grains"""

        with patch(
            "salt.proxy.nxos.sendline", autospec=True, return_value=n9k_show_ver_list[0]
        ):
            result = nxos_proxy.grains()
            self.assertEqual(result, n9k_grains)

    def test_sendline(self):

        """UT: nxos module:sendline method - nxapi"""

        command = "show version"

        with patch("salt.proxy.nxos._sendline_ssh", autospec=True) as sendline_ssh:
            result = nxos_proxy.sendline(command)
            self.assertEqual(result, sendline_ssh.return_value)

    def test_proxy_config(self):

        """UT: nxos module:proxy_config method - ssh success path"""

        commands = ["feature bgp", "router bgp 65535"]

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"save_config": False}):
            with patch("salt.proxy.nxos._sendline_ssh", autospec=True) as sendline_ssh:
                result = nxos_proxy.proxy_config(commands)
                self.assertEqual(result, [commands, sendline_ssh.return_value])

    def test_proxy_config_save_config(self):

        """UT: nxos module:proxy_config method - ssh success path"""

        commands = ["feature bgp", "router bgp 65535"]

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"save_config": None}):
            with patch("salt.proxy.nxos._sendline_ssh", autospec=True) as sendline_ssh:
                result = nxos_proxy.proxy_config(commands, save_config=True)
                self.assertEqual(result, [commands, sendline_ssh.return_value])

    def test_proxy_config_error(self):

        """UT: nxos module:proxy_config method - CommandExecutionError"""

        with patch(
            "salt.proxy.nxos._sendline_ssh",
            autospec=True,
            side_effect=CommandExecutionError,
        ):
            with self.assertRaises(CommandExecutionError):
                nxos_proxy.proxy_config("show version", save_config=True)

    def test__init_ssh_device_details(self):
        with patch("salt.proxy.nxos.SSHConnection", autospec=True) as SSHConnection:
            SSHConnection().sendline.return_value = ["", ""]

            with patch("salt.proxy.nxos.DEVICE_DETAILS", {}) as device_details:
                nxos_proxy._init_ssh(None)
                self.assertIn(nxos_proxy._worker_name(), device_details)
                self.assertTrue(device_details["initialized"])
                self.assertTrue(device_details["save_config"])

            with patch.dict(nxos_proxy.__opts__["proxy"], {"save_config": False}):
                with patch("salt.proxy.nxos.DEVICE_DETAILS", {}) as device_details:
                    nxos_proxy._init_ssh(None)
                    self.assertIn(nxos_proxy._worker_name(), device_details)
                    self.assertTrue(device_details["initialized"])
                    self.assertFalse(device_details["save_config"])

    def test__init_ssh_opts(self):

        """UT: nxos module:_init_ssh method - successful connectinon"""

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {}):
            with patch("salt.proxy.nxos.SSHConnection", autospec=True) as SSHConnection:
                SSHConnection().sendline.return_value = ["", ""]
                nxos_proxy._init_ssh(None)
                self.assertEqual(
                    nxos_proxy.__opts__["proxy"]["host"],
                    SSHConnection.call_args[1]["host"],
                )

                opts = MagicMock()
                nxos_proxy._init_ssh(opts)
                self.assertEqual(
                    opts["proxy"]["host"], SSHConnection.call_args[1]["host"]
                )

    def test__init_ssh_prompt(self):

        """UT: nxos module:_init_ssh method - prompt regex"""

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {}):
            with patch("salt.proxy.nxos.SSHConnection", autospec=True) as SSHConnection:
                SSHConnection().sendline.return_value = ["", ""]

                with patch.dict(
                    nxos_proxy.__opts__["proxy"], {"prompt_regex": "n9k.*device"}
                ):
                    nxos_proxy._init_ssh(None)
                    self.assertEqual(
                        "n9k.*device", SSHConnection.call_args[1]["prompt"]
                    )

                with patch.dict(
                    nxos_proxy.__opts__["proxy"], {"prompt_name": "n9k-device"}
                ):
                    nxos_proxy._init_ssh(None)
                    self.assertEqual(
                        "n9k-device.*#", SSHConnection.call_args[1]["prompt"]
                    )

                nxos_proxy._init_ssh(None)
                self.assertEqual(".+#$", SSHConnection.call_args[1]["prompt"])

    def test__initialized_ssh(self):

        """UT: nxos module:_initialized_ssh method"""

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {"initialized": True}):
            result = nxos_proxy._initialized_ssh()
            self.assertTrue(result)

        with patch("salt.proxy.nxos.DEVICE_DETAILS", {}):
            result = nxos_proxy._initialized_ssh()
            self.assertFalse(result)

    def test__parse_output_for_errors(self):

        """UT: nxos module:_parse_output_for_errors method"""

        data = "% Incomplete command at '^' marker."
        command = "show"

        with self.assertRaises(CommandExecutionError):
            nxos_proxy._parse_output_for_errors(
                data, command, error_pattern="Incomplete"
            )

        data = "% Incomplete command at '^' marker."
        command = "show"

        with self.assertRaises(CommandExecutionError):
            nxos_proxy._parse_output_for_errors(
                data, command, error_pattern=["Incomplete", "marker"]
            )

        data = "% Invalid command at '^' marker."
        command = "show bep"

        with self.assertRaises(CommandExecutionError):
            nxos_proxy._parse_output_for_errors(data, command)

        data = "% Incomplete command at '^' marker."
        command = "show"

        nxos_proxy._parse_output_for_errors(data, command)

        data = "% Incomplete command at '^' marker."
        command = "show"

        nxos_proxy._parse_output_for_errors(data, command, error_pattern="foo")

    def test__init_ssh_raise_exception(self):

        """UT: nxos module:_init_ssh method - raise exception"""

        class SSHException(Exception):
            pass

        with patch("salt.proxy.nxos.SSHConnection", autospec=True) as SSHConnection:
            with patch("salt.proxy.nxos.log", autospec=True) as log:
                with self.assertRaises(SSHException):
                    SSHConnection.side_effect = SSHException
                    nxos_proxy._init_ssh(None)
                log.error.assert_called()



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""


import errno

import salt.modules.grub_legacy as grub_legacy
import salt.utils.stringutils
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class GrublegacyTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.grub_legacy
    """

    def setup_loader_modules(self):
        return {grub_legacy: {}}

    def test_version(self):
        """
        Test for Return server version from grub --version
        """
        mock = MagicMock(return_value="out")
        with patch.dict(grub_legacy.__salt__, {"cmd.run": mock}):
            self.assertEqual(grub_legacy.version(), "out")

    def test_conf(self):
        """
        Test for Parse GRUB conf file
        """
        file_data = IOError(errno.EACCES, "Permission denied")
        with patch(
            "salt.utils.files.fopen", mock_open(read_data=file_data)
        ), patch.object(grub_legacy, "_detect_conf", return_value="A"):
            self.assertRaises(CommandExecutionError, grub_legacy.conf)

        file_data = salt.utils.stringutils.to_str("\n".join(["#", "A B C D,E,F G H"]))
        with patch(
            "salt.utils.files.fopen", mock_open(read_data=file_data)
        ), patch.object(grub_legacy, "_detect_conf", return_value="A"):
            conf = grub_legacy.conf()
            assert conf == {"A": "B C D,E,F G H", "stanzas": []}, conf



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest
import salt.states.boto_cloudwatch_alarm as boto_cloudwatch_alarm
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {boto_cloudwatch_alarm: {}}


def test_present():
    """
    Test to ensure the cloudwatch alarm exists.
    """
    name = "my test alarm"
    attributes = {
        "metric": "ApproximateNumberOfMessagesVisible",
        "namespace": "AWS/SQS",
    }

    ret = {"name": name, "result": None, "changes": {}, "comment": ""}

    mock = MagicMock(side_effect=[["ok_actions"], [], []])
    mock_bool = MagicMock(return_value=True)
    with patch.dict(
        boto_cloudwatch_alarm.__salt__,
        {
            "boto_cloudwatch.get_alarm": mock,
            "boto_cloudwatch.create_or_update_alarm": mock_bool,
        },
    ):
        with patch.dict(boto_cloudwatch_alarm.__opts__, {"test": True}):
            comt = "alarm my test alarm is to be created/updated."
            ret.update({"comment": comt})
            assert boto_cloudwatch_alarm.present(name, attributes) == ret

            comt = "alarm my test alarm is to be created/updated."
            ret.update({"comment": comt})
            assert boto_cloudwatch_alarm.present(name, attributes) == ret

        with patch.dict(boto_cloudwatch_alarm.__opts__, {"test": False}):
            changes = {
                "new": {
                    "metric": "ApproximateNumberOfMessagesVisible",
                    "namespace": "AWS/SQS",
                }
            }
            comt = "alarm my test alarm is to be created/updated."
            ret.update({"changes": changes, "comment": "", "result": True})
            assert boto_cloudwatch_alarm.present(name, attributes) == ret


def test_absent():
    """
    Test to ensure the named cloudwatch alarm is deleted.
    """
    name = "my test alarm"

    ret = {"name": name, "result": None, "changes": {}, "comment": ""}

    mock = MagicMock(side_effect=[True, False])
    with patch.dict(
        boto_cloudwatch_alarm.__salt__, {"boto_cloudwatch.get_alarm": mock}
    ):
        with patch.dict(boto_cloudwatch_alarm.__opts__, {"test": True}):
            comt = "alarm {} is set to be removed.".format(name)
            ret.update({"comment": comt})
            assert boto_cloudwatch_alarm.absent(name) == ret

            comt = "my test alarm does not exist in None."
            ret.update({"comment": comt, "result": True})
            assert boto_cloudwatch_alarm.absent(name) == ret



"""
Display no output
=================

No output is produced when this outputter is selected

CLI Example:

.. code-block:: bash

    salt '*' foo.bar --out=quiet
"""

# Define the module's virtual name
__virtualname__ = "quiet"


def __virtual__():
    return __virtualname__


def output(ret, **kwargs):  # pylint: disable=unused-argument
    """
    Don't display data. Used when you only are interested in the
    return.
    """
    return ""
